/* 
 * lesamnta_opt64.c
 *
 * Copyright (C) 2008 Hitachi, Ltd., All rights reserved.
 * Systems Development Laboratory, Hitachi, Ltd.
 */

#include <string.h>  /* for memset(), memcopy() */
#include "lesamnta_opt64.h"

# define BYTES_SWAP32(x)\
	((x << 24) | ((x & 0x0000ff00) << 8) | ((x & 0x00ff0000) >> 8) | (x >> 24))

# define BYTES_SWAP64(x)\
	(((u64)(BYTES_SWAP32((u32)(x))) << 32) | (u64)(BYTES_SWAP32((u32)((x) >> 32))))

/* BYTES_SWAP256(x) stores each 32-bit word of 256 bits data in little-endian convention */
#define BYTES_SWAP256(x) { \
	int _i = 8; while(_i--){x[_i] = BYTES_SWAP32(x[_i]);} \
}

/* BYTES_SWAP512(x) stores each 64-bit word of 512 bits data in little-endian convention */
#define BYTES_SWAP512(x) { \
	int _i = 8; while(_i--){x[_i] = BYTES_SWAP64(x[_i]);} \
}

#define ST224(x) (x->state.state256)
#define ST256(x) (x->state.state256)
#define ST384(x) (x->state.state512)
#define ST512(x) (x->state.state512)


#define ROUND256(a,b,c,d) \
	tmp.msg[0] = midmsg[a]     ^ midkey[b]; \
	tmp.msg[1] = midmsg[a + 1] ^ midkey[b + 1]; \
	tmp.key[0] = midkey[a] ^ (u32)CNS[d][0]; \
	tmp.key[1] = midkey[a + 1] ^ (u32)CNS[d][1]; \
	F256_1(tmp.key,tmp.t); \
	midkey[c]     ^= tmp.key[0]; \
	midkey[c + 1] ^= tmp.key[1]; \
	F256_2(tmp.msg, tmp.t); \
	midmsg[c]     ^= tmp.msg[0]; \
	midmsg[c + 1] ^= tmp.msg[1]; 

#define ROUND256_OUTPUT(a,b,c,d) \
	tmp.msg[0] = midmsg[a]     ^ midkey[b]; \
	tmp.msg[1] = midmsg[a + 1] ^ midkey[b + 1]; \
	tmp.key[0] = midkey[a] ^ (u32)CNS[d][0]; \
	tmp.key[1] = midkey[a + 1] ^ (u32)CNS[d][1]; \
	F256_2(tmp.key, tmp.t); \
	midkey[c]     ^= tmp.key[0]; \
	midkey[c + 1] ^= tmp.key[1]; \
	F256_2(tmp.msg, tmp.t); \
	midmsg[c]     ^= tmp.msg[0]; \
	midmsg[c + 1] ^= tmp.msg[1];

#define F256_1(s, t) \
	t[0] =  TBLB0[(s[0] >> 24)       ]; \
	t[0] ^= TBLB1[(s[0] >> 16) & 0xff]; \
	t[0] ^= TBLB2[(s[0] >>  8) & 0xff]; \
	t[0] ^= TBLB3[(s[0]      ) & 0xff]; \
	t[1] =  TBLB0[(s[1] >> 24)       ]; \
	t[1] ^= TBLB1[(s[1] >> 16) & 0xff]; \
	t[1] ^= TBLB2[(s[1] >>  8) & 0xff]; \
	t[1] ^= TBLB3[(s[1]      ) & 0xff]; \
	s[0] = (t[0] & 0xffff) | (t[1] & 0xffff0000); \
	s[1] = (t[1] & 0xffff) | (t[0] & 0xffff0000); 

#define F256_2(s, t) \
	t[0] = TBLA0[s[0] >> 24] ^ TBLA1[s[0] & 0xff] ^ TBLA2[(s[0] >> 8) & 0xff] ^ TBLA3[(s[1] >> 16) & 0xff]; \
	t[1] = TBLA0[s[1] >> 24] ^ TBLA1[s[1] & 0xff] ^ TBLA2[(s[1] >> 8) & 0xff] ^ TBLA3[(s[0] >> 16) & 0xff]; \
	s[0] = TBLA0[t[0] >> 24] ^ TBLA1[t[0] & 0xff] ^ TBLA2[(t[0] >> 8) & 0xff] ^ TBLA3[(t[1] >> 16) & 0xff]; \
	s[1] = TBLA0[t[1] >> 24] ^ TBLA1[t[1] & 0xff] ^ TBLA2[(t[1] >> 8) & 0xff] ^ TBLA3[(t[0] >> 16) & 0xff]; \
	t[0] = TBLA0[s[0] >> 24] ^ TBLA1[s[0] & 0xff] ^ TBLA2[(s[0] >> 8) & 0xff] ^ TBLA3[(s[1] >> 16) & 0xff]; \
	t[1] = TBLA0[s[1] >> 24] ^ TBLA1[s[1] & 0xff] ^ TBLA2[(s[1] >> 8) & 0xff] ^ TBLA3[(s[0] >> 16) & 0xff]; \
	s[0] = TBLA0[t[0] >> 24] ^ TBLA1[t[0] & 0xff] ^ TBLA2[(t[0] >> 8) & 0xff] ^ TBLA3[(t[1] >> 16) & 0xff]; \
	s[1] = TBLA0[t[1] >> 24] ^ TBLA1[t[1] & 0xff] ^ TBLA2[(t[1] >> 8) & 0xff] ^ TBLA3[(t[0] >> 16) & 0xff];

#define ROUND512(a, b, c, d) \
	tmpkey[0] = midkey[a] ^ (u64)CNS[d][0]; \
	tmpkey[1] = midkey[a + 1] ^ (u64)CNS[d][1]; \
	tmpmsg[0] = midmsg[a]     ^ midkey[b]; \
	tmpmsg[1] = midmsg[a + 1] ^ midkey[b + 1]; \
	F512_1(tmpkey, t); \
	midkey[c]     ^= tmpkey[0]; \
	midkey[c + 1] ^= tmpkey[1]; \
	F512_2(tmpmsg, t); \
	midmsg[c]     ^= tmpmsg[0]; \
	midmsg[c + 1] ^= tmpmsg[1];

#define ROUND512_OUTPUT(a, b, c, d) \
	tmpkey[0] = midkey[a] ^ (u64)CNS[d][0]; \
	tmpkey[1] = midkey[a + 1] ^ (u64)CNS[d][1]; \
	tmpmsg[0] = midmsg[a]     ^ midkey[b]; \
	tmpmsg[1] = midmsg[a + 1] ^ midkey[b + 1]; \
	F512_2(tmpkey, t); \
	midkey[c]     ^= tmpkey[0]; \
	midkey[c + 1] ^= tmpkey[1]; \
	F512_2(tmpmsg, t); \
	midmsg[c]     ^= tmpmsg[0]; \
	midmsg[c + 1] ^= tmpmsg[1];

#define F512_1(s, t) \
	t[0] = TBLC0[ s[0] >> 56] ^ \
	       TBLC1[(s[0] >> 48) & 0xff] ^ \
	       TBLC2[(s[0] >> 40) & 0xff] ^ \
	       TBLC3[(s[0] >> 32) & 0xff] ^ \
	       TBLC4[(s[0] >> 24) & 0xff] ^ \
	       TBLC5[(s[0] >> 16) & 0xff] ^ \
	       TBLC6[(s[0] >>  8) & 0xff] ^ \
	       TBLC7[ s[0]        & 0xff]; \
	t[1] = TBLC0[ s[1] >> 56] ^ \
	       TBLC1[(s[1] >> 48) & 0xff] ^ \
	       TBLC2[(s[1] >> 40) & 0xff] ^ \
	       TBLC3[(s[1] >> 32) & 0xff] ^ \
	       TBLC4[(s[1] >> 24) & 0xff] ^ \
	       TBLC5[(s[1] >> 16) & 0xff] ^ \
	       TBLC6[(s[1] >>  8) & 0xff] ^ \
	       TBLC7[ s[1]        & 0xff]; \
	s[0] = (t[1] & 0xffffffff00000000LL) | (t[0] & 0x00000000ffffffffLL); \
	s[1] = (t[0] & 0xffffffff00000000LL) | (t[1] & 0x00000000ffffffffLL);

#define F512_2(s, t) \
	t[0] = ((u64*)TBLD0)[ s[0] >> 56        ] ^ \
	       ((u64*)TBLD1)[(s[0] >> 16) & 0xff] ^ \
	       ((u64*)TBLD2)[(s[1] >> 40) & 0xff] ^ \
	       ((u64*)TBLD3)[ s[1]        & 0xff] ^ \
	       TBLD0[((s[0] >> 24) & 0xff) * 2 + 1] ^ \
	       TBLD1[((s[1] >> 48) & 0xff) * 2 + 1] ^ \
	       TBLD2[((s[1] >>  8) & 0xff) * 2 + 1] ^ \
	       TBLD3[((s[0] >> 32) & 0xff) * 2 + 1]; \
	t[1] = ((u64*)TBLD0)[ s[1] >> 56        ] ^ \
	       ((u64*)TBLD1)[(s[1] >> 16) & 0xff] ^ \
	       ((u64*)TBLD2)[(s[0] >> 40) & 0xff] ^ \
	       ((u64*)TBLD3)[ s[0]        & 0xff] ^ \
	       TBLD0[((s[1] >> 24) & 0xff) * 2 + 1] ^ \
	       TBLD1[((s[0] >> 48) & 0xff) * 2 + 1] ^ \
	       TBLD2[((s[0] >>  8) & 0xff) * 2 + 1] ^ \
	       TBLD3[((s[1] >> 32) & 0xff) * 2 + 1]; \
	s[0] = ((u64*)TBLD0)[ t[0] >> 56        ] ^ \
	       ((u64*)TBLD1)[(t[0] >> 16) & 0xff] ^ \
	       ((u64*)TBLD2)[(t[1] >> 40) & 0xff] ^ \
	       ((u64*)TBLD3)[ t[1]        & 0xff] ^ \
	       TBLD0[((t[0] >> 24) & 0xff) * 2 + 1] ^ \
	       TBLD1[((t[1] >> 48) & 0xff) * 2 + 1] ^ \
	       TBLD2[((t[1] >>  8) & 0xff) * 2 + 1] ^ \
	       TBLD3[((t[0] >> 32) & 0xff) * 2 + 1]; \
	s[1] = ((u64*)TBLD0)[ t[1] >> 56        ] ^ \
	       ((u64*)TBLD1)[(t[1] >> 16) & 0xff] ^ \
	       ((u64*)TBLD2)[(t[0] >> 40) & 0xff] ^ \
	       ((u64*)TBLD3)[ t[0]        & 0xff] ^ \
	       TBLD0[((t[1] >> 24) & 0xff) * 2 + 1] ^ \
	       TBLD1[((t[0] >> 48) & 0xff) * 2 + 1] ^ \
	       TBLD2[((t[0] >>  8) & 0xff) * 2 + 1] ^ \
	       TBLD3[((t[1] >> 32) & 0xff) * 2 + 1]; \
	t[0] = ((u64*)TBLD0)[ s[0] >> 56        ] ^ \
	       ((u64*)TBLD1)[(s[0] >> 16) & 0xff] ^ \
	       ((u64*)TBLD2)[(s[1] >> 40) & 0xff] ^ \
	       ((u64*)TBLD3)[ s[1]        & 0xff] ^ \
	       TBLD0[((s[0] >> 24) & 0xff) * 2 + 1] ^ \
	       TBLD1[((s[1] >> 48) & 0xff) * 2 + 1] ^ \
	       TBLD2[((s[1] >>  8) & 0xff) * 2 + 1] ^ \
	       TBLD3[((s[0] >> 32) & 0xff) * 2 + 1]; \
	t[1] = ((u64*)TBLD0)[ s[1] >> 56        ] ^ \
	       ((u64*)TBLD1)[(s[1] >> 16) & 0xff] ^ \
	       ((u64*)TBLD2)[(s[0] >> 40) & 0xff] ^ \
	       ((u64*)TBLD3)[ s[0]        & 0xff] ^ \
	       TBLD0[((s[1] >> 24) & 0xff) * 2 + 1] ^ \
	       TBLD1[((s[0] >> 48) & 0xff) * 2 + 1] ^ \
	       TBLD2[((s[0] >>  8) & 0xff) * 2 + 1] ^ \
	       TBLD3[((s[1] >> 32) & 0xff) * 2 + 1]; \
	s[0] = ((u64*)TBLD0)[ t[0] >> 56        ] ^ \
	       ((u64*)TBLD1)[(t[0] >> 16) & 0xff] ^ \
	       ((u64*)TBLD2)[(t[1] >> 40) & 0xff] ^ \
	       ((u64*)TBLD3)[ t[1]        & 0xff] ^ \
	       TBLD0[((t[0] >> 24) & 0xff) * 2 + 1] ^ \
	       TBLD1[((t[1] >> 48) & 0xff) * 2 + 1] ^ \
	       TBLD2[((t[1] >>  8) & 0xff) * 2 + 1] ^ \
	       TBLD3[((t[0] >> 32) & 0xff) * 2 + 1]; \
	s[1] = ((u64*)TBLD0)[ t[1] >> 56        ] ^ \
	       ((u64*)TBLD1)[(t[1] >> 16) & 0xff] ^ \
	       ((u64*)TBLD2)[(t[0] >> 40) & 0xff] ^ \
	       ((u64*)TBLD3)[ t[0]        & 0xff] ^ \
	       TBLD0[((t[1] >> 24) & 0xff) * 2 + 1] ^ \
	       TBLD1[((t[0] >> 48) & 0xff) * 2 + 1] ^ \
	       TBLD2[((t[0] >>  8) & 0xff) * 2 + 1] ^ \
	       TBLD3[((t[1] >> 32) & 0xff) * 2 + 1]; 

HashReturn Init256(hashState *state);
HashReturn Update256(hashState *state, const BitSequence *data, Datalength databitlen);
HashReturn Final256(hashState *state, BitSequence *hashval);

HashReturn Init512(hashState *state);
HashReturn Update512(hashState *state, const BitSequence *data, Datalength databitlen);
HashReturn Final512(hashState *state, BitSequence *hashval);

HashReturn Init224(hashState *state);
HashReturn Update224(hashState *state, const BitSequence *data, Datalength databitlen);
HashReturn Final224(hashState *state, BitSequence *hashval);

HashReturn Init384(hashState *state);
HashReturn Update384(hashState *state, const BitSequence *data, Datalength databitlen);
HashReturn Final384(hashState *state, BitSequence *hashval);

static void Output256(u32 *H, const u32 *M);
static void Compression256(u32 *H, const u32 *M);

static void Output512(u64 *H, const u64 *M);
static void Compression512(u64 *H, const u64 *M);

/* Initial hash velue of Lesamnta-224 */
static const u32 H_init224[8] = {
	0x00000224, 0x00000224, 0x00000224, 0x00000224, 
	0x00000224, 0x00000224, 0x00000224, 0x00000224
};

/* Initial hash velue of Lesamnta-256 */
static const u32 H_init256[8] = {
	0x00000256, 0x00000256, 0x00000256, 0x00000256, 
	0x00000256, 0x00000256, 0x00000256, 0x00000256
};

/* Initial hash velue of Lesamnta-384 */
static const u64 H_init384[8] = {
	0x0000000000000384LL, 0x0000000000000384LL, 0x0000000000000384LL, 0x0000000000000384LL, 
	0x0000000000000384LL, 0x0000000000000384LL, 0x0000000000000384LL, 0x0000000000000384LL
};

/* Initial hash velue of Lesamnta-512 */
static const u64 H_init512[8] = {
	0x0000000000000512LL, 0x0000000000000512LL, 0x0000000000000512LL, 0x0000000000000512LL, 
	0x0000000000000512LL, 0x0000000000000512LL, 0x0000000000000512LL, 0x0000000000000512LL
};

static const u32 TBLA0[256]={
	0xc6630000, 0xf87c0000, 0xee770000, 0xf67b0000, 
	0xfff20000, 0xd66b0000, 0xde6f0000, 0x91c50000, 
	0x60300000, 0x02010000, 0xce670000, 0x562b0000, 
	0xe7fe0000, 0xb5d70000, 0x4dab0000, 0xec760000, 
	0x8fca0000, 0x1f820000, 0x89c90000, 0xfa7d0000, 
	0xeffa0000, 0xb2590000, 0x8e470000, 0xfbf00000, 
	0x41ad0000, 0xb3d40000, 0x5fa20000, 0x45af0000, 
	0x239c0000, 0x53a40000, 0xe4720000, 0x9bc00000, 
	0x75b70000, 0xe1fd0000, 0x3d930000, 0x4c260000, 
	0x6c360000, 0x7e3f0000, 0xf5f70000, 0x83cc0000, 
	0x68340000, 0x51a50000, 0xd1e50000, 0xf9f10000, 
	0xe2710000, 0xabd80000, 0x62310000, 0x2a150000, 
	0x08040000, 0x95c70000, 0x46230000, 0x9dc30000, 
	0x30180000, 0x37960000, 0x0a050000, 0x2f9a0000, 
	0x0e070000, 0x24120000, 0x1b800000, 0xdfe20000, 
	0xcdeb0000, 0x4e270000, 0x7fb20000, 0xea750000, 
	0x12090000, 0x1d830000, 0x582c0000, 0x341a0000, 
	0x361b0000, 0xdc6e0000, 0xb45a0000, 0x5ba00000, 
	0xa4520000, 0x763b0000, 0xb7d60000, 0x7db30000, 
	0x52290000, 0xdde30000, 0x5e2f0000, 0x13840000, 
	0xa6530000, 0xb9d10000, 0x00000000, 0xc1ed0000, 
	0x40200000, 0xe3fc0000, 0x79b10000, 0xb65b0000, 
	0xd46a0000, 0x8dcb0000, 0x67be0000, 0x72390000, 
	0x944a0000, 0x984c0000, 0xb0580000, 0x85cf0000, 
	0xbbd00000, 0xc5ef0000, 0x4faa0000, 0xedfb0000, 
	0x86430000, 0x9a4d0000, 0x66330000, 0x11850000, 
	0x8a450000, 0xe9f90000, 0x04020000, 0xfe7f0000, 
	0xa0500000, 0x783c0000, 0x259f0000, 0x4ba80000, 
	0xa2510000, 0x5da30000, 0x80400000, 0x058f0000, 
	0x3f920000, 0x219d0000, 0x70380000, 0xf1f50000, 
	0x63bc0000, 0x77b60000, 0xafda0000, 0x42210000, 
	0x20100000, 0xe5ff0000, 0xfdf30000, 0xbfd20000, 
	0x81cd0000, 0x180c0000, 0x26130000, 0xc3ec0000, 
	0xbe5f0000, 0x35970000, 0x88440000, 0x2e170000, 
	0x93c40000, 0x55a70000, 0xfc7e0000, 0x7a3d0000, 
	0xc8640000, 0xba5d0000, 0x32190000, 0xe6730000, 
	0xc0600000, 0x19810000, 0x9e4f0000, 0xa3dc0000, 
	0x44220000, 0x542a0000, 0x3b900000, 0x0b880000, 
	0x8c460000, 0xc7ee0000, 0x6bb80000, 0x28140000, 
	0xa7de0000, 0xbc5e0000, 0x160b0000, 0xaddb0000, 
	0xdbe00000, 0x64320000, 0x743a0000, 0x140a0000, 
	0x92490000, 0x0c060000, 0x48240000, 0xb85c0000, 
	0x9fc20000, 0xbdd30000, 0x43ac0000, 0xc4620000, 
	0x39910000, 0x31950000, 0xd3e40000, 0xf2790000, 
	0xd5e70000, 0x8bc80000, 0x6e370000, 0xda6d0000, 
	0x018d0000, 0xb1d50000, 0x9c4e0000, 0x49a90000, 
	0xd86c0000, 0xac560000, 0xf3f40000, 0xcfea0000, 
	0xca650000, 0xf47a0000, 0x47ae0000, 0x10080000, 
	0x6fba0000, 0xf0780000, 0x4a250000, 0x5c2e0000, 
	0x381c0000, 0x57a60000, 0x73b40000, 0x97c60000, 
	0xcbe80000, 0xa1dd0000, 0xe8740000, 0x3e1f0000, 
	0x964b0000, 0x61bd0000, 0x0d8b0000, 0x0f8a0000, 
	0xe0700000, 0x7c3e0000, 0x71b50000, 0xcc660000, 
	0x90480000, 0x06030000, 0xf7f60000, 0x1c0e0000, 
	0xc2610000, 0x6a350000, 0xae570000, 0x69b90000, 
	0x17860000, 0x99c10000, 0x3a1d0000, 0x279e0000, 
	0xd9e10000, 0xebf80000, 0x2b980000, 0x22110000, 
	0xd2690000, 0xa9d90000, 0x078e0000, 0x33940000, 
	0x2d9b0000, 0x3c1e0000, 0x15870000, 0xc9e90000, 
	0x87ce0000, 0xaa550000, 0x50280000, 0xa5df0000, 
	0x038c0000, 0x59a10000, 0x09890000, 0x1a0d0000, 
	0x65bf0000, 0xd7e60000, 0x84420000, 0xd0680000, 
	0x82410000, 0x29990000, 0x5a2d0000, 0x1e0f0000, 
	0x7bb00000, 0xa8540000, 0x6dbb0000, 0x2c160000
};

static const u32 TBLA1[256]={
	0x63c60000, 0x7cf80000, 0x77ee0000, 0x7bf60000, 
	0xf2ff0000, 0x6bd60000, 0x6fde0000, 0xc5910000, 
	0x30600000, 0x01020000, 0x67ce0000, 0x2b560000, 
	0xfee70000, 0xd7b50000, 0xab4d0000, 0x76ec0000, 
	0xca8f0000, 0x821f0000, 0xc9890000, 0x7dfa0000, 
	0xfaef0000, 0x59b20000, 0x478e0000, 0xf0fb0000, 
	0xad410000, 0xd4b30000, 0xa25f0000, 0xaf450000, 
	0x9c230000, 0xa4530000, 0x72e40000, 0xc09b0000, 
	0xb7750000, 0xfde10000, 0x933d0000, 0x264c0000, 
	0x366c0000, 0x3f7e0000, 0xf7f50000, 0xcc830000, 
	0x34680000, 0xa5510000, 0xe5d10000, 0xf1f90000, 
	0x71e20000, 0xd8ab0000, 0x31620000, 0x152a0000, 
	0x04080000, 0xc7950000, 0x23460000, 0xc39d0000, 
	0x18300000, 0x96370000, 0x050a0000, 0x9a2f0000, 
	0x070e0000, 0x12240000, 0x801b0000, 0xe2df0000, 
	0xebcd0000, 0x274e0000, 0xb27f0000, 0x75ea0000, 
	0x09120000, 0x831d0000, 0x2c580000, 0x1a340000, 
	0x1b360000, 0x6edc0000, 0x5ab40000, 0xa05b0000, 
	0x52a40000, 0x3b760000, 0xd6b70000, 0xb37d0000, 
	0x29520000, 0xe3dd0000, 0x2f5e0000, 0x84130000, 
	0x53a60000, 0xd1b90000, 0x00000000, 0xedc10000, 
	0x20400000, 0xfce30000, 0xb1790000, 0x5bb60000, 
	0x6ad40000, 0xcb8d0000, 0xbe670000, 0x39720000, 
	0x4a940000, 0x4c980000, 0x58b00000, 0xcf850000, 
	0xd0bb0000, 0xefc50000, 0xaa4f0000, 0xfbed0000, 
	0x43860000, 0x4d9a0000, 0x33660000, 0x85110000, 
	0x458a0000, 0xf9e90000, 0x02040000, 0x7ffe0000, 
	0x50a00000, 0x3c780000, 0x9f250000, 0xa84b0000, 
	0x51a20000, 0xa35d0000, 0x40800000, 0x8f050000, 
	0x923f0000, 0x9d210000, 0x38700000, 0xf5f10000, 
	0xbc630000, 0xb6770000, 0xdaaf0000, 0x21420000, 
	0x10200000, 0xffe50000, 0xf3fd0000, 0xd2bf0000, 
	0xcd810000, 0x0c180000, 0x13260000, 0xecc30000, 
	0x5fbe0000, 0x97350000, 0x44880000, 0x172e0000, 
	0xc4930000, 0xa7550000, 0x7efc0000, 0x3d7a0000, 
	0x64c80000, 0x5dba0000, 0x19320000, 0x73e60000, 
	0x60c00000, 0x81190000, 0x4f9e0000, 0xdca30000, 
	0x22440000, 0x2a540000, 0x903b0000, 0x880b0000, 
	0x468c0000, 0xeec70000, 0xb86b0000, 0x14280000, 
	0xdea70000, 0x5ebc0000, 0x0b160000, 0xdbad0000, 
	0xe0db0000, 0x32640000, 0x3a740000, 0x0a140000, 
	0x49920000, 0x060c0000, 0x24480000, 0x5cb80000, 
	0xc29f0000, 0xd3bd0000, 0xac430000, 0x62c40000, 
	0x91390000, 0x95310000, 0xe4d30000, 0x79f20000, 
	0xe7d50000, 0xc88b0000, 0x376e0000, 0x6dda0000, 
	0x8d010000, 0xd5b10000, 0x4e9c0000, 0xa9490000, 
	0x6cd80000, 0x56ac0000, 0xf4f30000, 0xeacf0000, 
	0x65ca0000, 0x7af40000, 0xae470000, 0x08100000, 
	0xba6f0000, 0x78f00000, 0x254a0000, 0x2e5c0000, 
	0x1c380000, 0xa6570000, 0xb4730000, 0xc6970000, 
	0xe8cb0000, 0xdda10000, 0x74e80000, 0x1f3e0000, 
	0x4b960000, 0xbd610000, 0x8b0d0000, 0x8a0f0000, 
	0x70e00000, 0x3e7c0000, 0xb5710000, 0x66cc0000, 
	0x48900000, 0x03060000, 0xf6f70000, 0x0e1c0000, 
	0x61c20000, 0x356a0000, 0x57ae0000, 0xb9690000, 
	0x86170000, 0xc1990000, 0x1d3a0000, 0x9e270000, 
	0xe1d90000, 0xf8eb0000, 0x982b0000, 0x11220000, 
	0x69d20000, 0xd9a90000, 0x8e070000, 0x94330000, 
	0x9b2d0000, 0x1e3c0000, 0x87150000, 0xe9c90000, 
	0xce870000, 0x55aa0000, 0x28500000, 0xdfa50000, 
	0x8c030000, 0xa1590000, 0x89090000, 0x0d1a0000, 
	0xbf650000, 0xe6d70000, 0x42840000, 0x68d00000, 
	0x41820000, 0x99290000, 0x2d5a0000, 0x0f1e0000, 
	0xb07b0000, 0x54a80000, 0xbb6d0000, 0x162c0000
};

static const u32 TBLA2[256]={
	0x0000c663, 0x0000f87c, 0x0000ee77, 0x0000f67b, 
	0x0000fff2, 0x0000d66b, 0x0000de6f, 0x000091c5, 
	0x00006030, 0x00000201, 0x0000ce67, 0x0000562b, 
	0x0000e7fe, 0x0000b5d7, 0x00004dab, 0x0000ec76, 
	0x00008fca, 0x00001f82, 0x000089c9, 0x0000fa7d, 
	0x0000effa, 0x0000b259, 0x00008e47, 0x0000fbf0, 
	0x000041ad, 0x0000b3d4, 0x00005fa2, 0x000045af, 
	0x0000239c, 0x000053a4, 0x0000e472, 0x00009bc0, 
	0x000075b7, 0x0000e1fd, 0x00003d93, 0x00004c26, 
	0x00006c36, 0x00007e3f, 0x0000f5f7, 0x000083cc, 
	0x00006834, 0x000051a5, 0x0000d1e5, 0x0000f9f1, 
	0x0000e271, 0x0000abd8, 0x00006231, 0x00002a15, 
	0x00000804, 0x000095c7, 0x00004623, 0x00009dc3, 
	0x00003018, 0x00003796, 0x00000a05, 0x00002f9a, 
	0x00000e07, 0x00002412, 0x00001b80, 0x0000dfe2, 
	0x0000cdeb, 0x00004e27, 0x00007fb2, 0x0000ea75, 
	0x00001209, 0x00001d83, 0x0000582c, 0x0000341a, 
	0x0000361b, 0x0000dc6e, 0x0000b45a, 0x00005ba0, 
	0x0000a452, 0x0000763b, 0x0000b7d6, 0x00007db3, 
	0x00005229, 0x0000dde3, 0x00005e2f, 0x00001384, 
	0x0000a653, 0x0000b9d1, 0x00000000, 0x0000c1ed, 
	0x00004020, 0x0000e3fc, 0x000079b1, 0x0000b65b, 
	0x0000d46a, 0x00008dcb, 0x000067be, 0x00007239, 
	0x0000944a, 0x0000984c, 0x0000b058, 0x000085cf, 
	0x0000bbd0, 0x0000c5ef, 0x00004faa, 0x0000edfb, 
	0x00008643, 0x00009a4d, 0x00006633, 0x00001185, 
	0x00008a45, 0x0000e9f9, 0x00000402, 0x0000fe7f, 
	0x0000a050, 0x0000783c, 0x0000259f, 0x00004ba8, 
	0x0000a251, 0x00005da3, 0x00008040, 0x0000058f, 
	0x00003f92, 0x0000219d, 0x00007038, 0x0000f1f5, 
	0x000063bc, 0x000077b6, 0x0000afda, 0x00004221, 
	0x00002010, 0x0000e5ff, 0x0000fdf3, 0x0000bfd2, 
	0x000081cd, 0x0000180c, 0x00002613, 0x0000c3ec, 
	0x0000be5f, 0x00003597, 0x00008844, 0x00002e17, 
	0x000093c4, 0x000055a7, 0x0000fc7e, 0x00007a3d, 
	0x0000c864, 0x0000ba5d, 0x00003219, 0x0000e673, 
	0x0000c060, 0x00001981, 0x00009e4f, 0x0000a3dc, 
	0x00004422, 0x0000542a, 0x00003b90, 0x00000b88, 
	0x00008c46, 0x0000c7ee, 0x00006bb8, 0x00002814, 
	0x0000a7de, 0x0000bc5e, 0x0000160b, 0x0000addb, 
	0x0000dbe0, 0x00006432, 0x0000743a, 0x0000140a, 
	0x00009249, 0x00000c06, 0x00004824, 0x0000b85c, 
	0x00009fc2, 0x0000bdd3, 0x000043ac, 0x0000c462, 
	0x00003991, 0x00003195, 0x0000d3e4, 0x0000f279, 
	0x0000d5e7, 0x00008bc8, 0x00006e37, 0x0000da6d, 
	0x0000018d, 0x0000b1d5, 0x00009c4e, 0x000049a9, 
	0x0000d86c, 0x0000ac56, 0x0000f3f4, 0x0000cfea, 
	0x0000ca65, 0x0000f47a, 0x000047ae, 0x00001008, 
	0x00006fba, 0x0000f078, 0x00004a25, 0x00005c2e, 
	0x0000381c, 0x000057a6, 0x000073b4, 0x000097c6, 
	0x0000cbe8, 0x0000a1dd, 0x0000e874, 0x00003e1f, 
	0x0000964b, 0x000061bd, 0x00000d8b, 0x00000f8a, 
	0x0000e070, 0x00007c3e, 0x000071b5, 0x0000cc66, 
	0x00009048, 0x00000603, 0x0000f7f6, 0x00001c0e, 
	0x0000c261, 0x00006a35, 0x0000ae57, 0x000069b9, 
	0x00001786, 0x000099c1, 0x00003a1d, 0x0000279e, 
	0x0000d9e1, 0x0000ebf8, 0x00002b98, 0x00002211, 
	0x0000d269, 0x0000a9d9, 0x0000078e, 0x00003394, 
	0x00002d9b, 0x00003c1e, 0x00001587, 0x0000c9e9, 
	0x000087ce, 0x0000aa55, 0x00005028, 0x0000a5df, 
	0x0000038c, 0x000059a1, 0x00000989, 0x00001a0d, 
	0x000065bf, 0x0000d7e6, 0x00008442, 0x0000d068, 
	0x00008241, 0x00002999, 0x00005a2d, 0x00001e0f, 
	0x00007bb0, 0x0000a854, 0x00006dbb, 0x00002c16
};

static const u32 TBLA3[256]={
	0x000063c6, 0x00007cf8, 0x000077ee, 0x00007bf6, 
	0x0000f2ff, 0x00006bd6, 0x00006fde, 0x0000c591, 
	0x00003060, 0x00000102, 0x000067ce, 0x00002b56, 
	0x0000fee7, 0x0000d7b5, 0x0000ab4d, 0x000076ec, 
	0x0000ca8f, 0x0000821f, 0x0000c989, 0x00007dfa, 
	0x0000faef, 0x000059b2, 0x0000478e, 0x0000f0fb, 
	0x0000ad41, 0x0000d4b3, 0x0000a25f, 0x0000af45, 
	0x00009c23, 0x0000a453, 0x000072e4, 0x0000c09b, 
	0x0000b775, 0x0000fde1, 0x0000933d, 0x0000264c, 
	0x0000366c, 0x00003f7e, 0x0000f7f5, 0x0000cc83, 
	0x00003468, 0x0000a551, 0x0000e5d1, 0x0000f1f9, 
	0x000071e2, 0x0000d8ab, 0x00003162, 0x0000152a, 
	0x00000408, 0x0000c795, 0x00002346, 0x0000c39d, 
	0x00001830, 0x00009637, 0x0000050a, 0x00009a2f, 
	0x0000070e, 0x00001224, 0x0000801b, 0x0000e2df, 
	0x0000ebcd, 0x0000274e, 0x0000b27f, 0x000075ea, 
	0x00000912, 0x0000831d, 0x00002c58, 0x00001a34, 
	0x00001b36, 0x00006edc, 0x00005ab4, 0x0000a05b, 
	0x000052a4, 0x00003b76, 0x0000d6b7, 0x0000b37d, 
	0x00002952, 0x0000e3dd, 0x00002f5e, 0x00008413, 
	0x000053a6, 0x0000d1b9, 0x00000000, 0x0000edc1, 
	0x00002040, 0x0000fce3, 0x0000b179, 0x00005bb6, 
	0x00006ad4, 0x0000cb8d, 0x0000be67, 0x00003972, 
	0x00004a94, 0x00004c98, 0x000058b0, 0x0000cf85, 
	0x0000d0bb, 0x0000efc5, 0x0000aa4f, 0x0000fbed, 
	0x00004386, 0x00004d9a, 0x00003366, 0x00008511, 
	0x0000458a, 0x0000f9e9, 0x00000204, 0x00007ffe, 
	0x000050a0, 0x00003c78, 0x00009f25, 0x0000a84b, 
	0x000051a2, 0x0000a35d, 0x00004080, 0x00008f05, 
	0x0000923f, 0x00009d21, 0x00003870, 0x0000f5f1, 
	0x0000bc63, 0x0000b677, 0x0000daaf, 0x00002142, 
	0x00001020, 0x0000ffe5, 0x0000f3fd, 0x0000d2bf, 
	0x0000cd81, 0x00000c18, 0x00001326, 0x0000ecc3, 
	0x00005fbe, 0x00009735, 0x00004488, 0x0000172e, 
	0x0000c493, 0x0000a755, 0x00007efc, 0x00003d7a, 
	0x000064c8, 0x00005dba, 0x00001932, 0x000073e6, 
	0x000060c0, 0x00008119, 0x00004f9e, 0x0000dca3, 
	0x00002244, 0x00002a54, 0x0000903b, 0x0000880b, 
	0x0000468c, 0x0000eec7, 0x0000b86b, 0x00001428, 
	0x0000dea7, 0x00005ebc, 0x00000b16, 0x0000dbad, 
	0x0000e0db, 0x00003264, 0x00003a74, 0x00000a14, 
	0x00004992, 0x0000060c, 0x00002448, 0x00005cb8, 
	0x0000c29f, 0x0000d3bd, 0x0000ac43, 0x000062c4, 
	0x00009139, 0x00009531, 0x0000e4d3, 0x000079f2, 
	0x0000e7d5, 0x0000c88b, 0x0000376e, 0x00006dda, 
	0x00008d01, 0x0000d5b1, 0x00004e9c, 0x0000a949, 
	0x00006cd8, 0x000056ac, 0x0000f4f3, 0x0000eacf, 
	0x000065ca, 0x00007af4, 0x0000ae47, 0x00000810, 
	0x0000ba6f, 0x000078f0, 0x0000254a, 0x00002e5c, 
	0x00001c38, 0x0000a657, 0x0000b473, 0x0000c697, 
	0x0000e8cb, 0x0000dda1, 0x000074e8, 0x00001f3e, 
	0x00004b96, 0x0000bd61, 0x00008b0d, 0x00008a0f, 
	0x000070e0, 0x00003e7c, 0x0000b571, 0x000066cc, 
	0x00004890, 0x00000306, 0x0000f6f7, 0x00000e1c, 
	0x000061c2, 0x0000356a, 0x000057ae, 0x0000b969, 
	0x00008617, 0x0000c199, 0x00001d3a, 0x00009e27, 
	0x0000e1d9, 0x0000f8eb, 0x0000982b, 0x00001122, 
	0x000069d2, 0x0000d9a9, 0x00008e07, 0x00009433, 
	0x00009b2d, 0x00001e3c, 0x00008715, 0x0000e9c9, 
	0x0000ce87, 0x000055aa, 0x00002850, 0x0000dfa5, 
	0x00008c03, 0x0000a159, 0x00008909, 0x00000d1a, 
	0x0000bf65, 0x0000e6d7, 0x00004284, 0x000068d0, 
	0x00004182, 0x00009929, 0x00002d5a, 0x00000f1e, 
	0x0000b07b, 0x000054a8, 0x0000bb6d, 0x0000162c
};

static const u32 TBLB0[256]={
	0xc66363a5, 0xf87c7c84, 0xee777799, 0xf67b7b8d, 
	0xfff2f20d, 0xd66b6bbd, 0xde6f6fb1, 0x91c5c554, 
	0x60303050, 0x02010103, 0xce6767a9, 0x562b2b7d, 
	0xe7fefe19, 0xb5d7d762, 0x4dababe6, 0xec76769a, 
	0x8fcaca45, 0x1f82829d, 0x89c9c940, 0xfa7d7d87, 
	0xeffafa15, 0xb25959eb, 0x8e4747c9, 0xfbf0f00b, 
	0x41adadec, 0xb3d4d467, 0x5fa2a2fd, 0x45afafea, 
	0x239c9cbf, 0x53a4a4f7, 0xe4727296, 0x9bc0c05b, 
	0x75b7b7c2, 0xe1fdfd1c, 0x3d9393ae, 0x4c26266a, 
	0x6c36365a, 0x7e3f3f41, 0xf5f7f702, 0x83cccc4f, 
	0x6834345c, 0x51a5a5f4, 0xd1e5e534, 0xf9f1f108, 
	0xe2717193, 0xabd8d873, 0x62313153, 0x2a15153f, 
	0x0804040c, 0x95c7c752, 0x46232365, 0x9dc3c35e, 
	0x30181828, 0x379696a1, 0x0a05050f, 0x2f9a9ab5, 
	0x0e070709, 0x24121236, 0x1b80809b, 0xdfe2e23d, 
	0xcdebeb26, 0x4e272769, 0x7fb2b2cd, 0xea75759f, 
	0x1209091b, 0x1d83839e, 0x582c2c74, 0x341a1a2e, 
	0x361b1b2d, 0xdc6e6eb2, 0xb45a5aee, 0x5ba0a0fb, 
	0xa45252f6, 0x763b3b4d, 0xb7d6d661, 0x7db3b3ce, 
	0x5229297b, 0xdde3e33e, 0x5e2f2f71, 0x13848497, 
	0xa65353f5, 0xb9d1d168, 0x00000000, 0xc1eded2c, 
	0x40202060, 0xe3fcfc1f, 0x79b1b1c8, 0xb65b5bed, 
	0xd46a6abe, 0x8dcbcb46, 0x67bebed9, 0x7239394b, 
	0x944a4ade, 0x984c4cd4, 0xb05858e8, 0x85cfcf4a, 
	0xbbd0d06b, 0xc5efef2a, 0x4faaaae5, 0xedfbfb16, 
	0x864343c5, 0x9a4d4dd7, 0x66333355, 0x11858594, 
	0x8a4545cf, 0xe9f9f910, 0x04020206, 0xfe7f7f81, 
	0xa05050f0, 0x783c3c44, 0x259f9fba, 0x4ba8a8e3, 
	0xa25151f3, 0x5da3a3fe, 0x804040c0, 0x058f8f8a, 
	0x3f9292ad, 0x219d9dbc, 0x70383848, 0xf1f5f504, 
	0x63bcbcdf, 0x77b6b6c1, 0xafdada75, 0x42212163, 
	0x20101030, 0xe5ffff1a, 0xfdf3f30e, 0xbfd2d26d, 
	0x81cdcd4c, 0x180c0c14, 0x26131335, 0xc3ecec2f, 
	0xbe5f5fe1, 0x359797a2, 0x884444cc, 0x2e171739, 
	0x93c4c457, 0x55a7a7f2, 0xfc7e7e82, 0x7a3d3d47, 
	0xc86464ac, 0xba5d5de7, 0x3219192b, 0xe6737395, 
	0xc06060a0, 0x19818198, 0x9e4f4fd1, 0xa3dcdc7f, 
	0x44222266, 0x542a2a7e, 0x3b9090ab, 0x0b888883, 
	0x8c4646ca, 0xc7eeee29, 0x6bb8b8d3, 0x2814143c, 
	0xa7dede79, 0xbc5e5ee2, 0x160b0b1d, 0xaddbdb76, 
	0xdbe0e03b, 0x64323256, 0x743a3a4e, 0x140a0a1e, 
	0x924949db, 0x0c06060a, 0x4824246c, 0xb85c5ce4, 
	0x9fc2c25d, 0xbdd3d36e, 0x43acacef, 0xc46262a6, 
	0x399191a8, 0x319595a4, 0xd3e4e437, 0xf279798b, 
	0xd5e7e732, 0x8bc8c843, 0x6e373759, 0xda6d6db7, 
	0x018d8d8c, 0xb1d5d564, 0x9c4e4ed2, 0x49a9a9e0, 
	0xd86c6cb4, 0xac5656fa, 0xf3f4f407, 0xcfeaea25, 
	0xca6565af, 0xf47a7a8e, 0x47aeaee9, 0x10080818, 
	0x6fbabad5, 0xf0787888, 0x4a25256f, 0x5c2e2e72, 
	0x381c1c24, 0x57a6a6f1, 0x73b4b4c7, 0x97c6c651, 
	0xcbe8e823, 0xa1dddd7c, 0xe874749c, 0x3e1f1f21, 
	0x964b4bdd, 0x61bdbddc, 0x0d8b8b86, 0x0f8a8a85, 
	0xe0707090, 0x7c3e3e42, 0x71b5b5c4, 0xcc6666aa, 
	0x904848d8, 0x06030305, 0xf7f6f601, 0x1c0e0e12, 
	0xc26161a3, 0x6a35355f, 0xae5757f9, 0x69b9b9d0, 
	0x17868691, 0x99c1c158, 0x3a1d1d27, 0x279e9eb9, 
	0xd9e1e138, 0xebf8f813, 0x2b9898b3, 0x22111133, 
	0xd26969bb, 0xa9d9d970, 0x078e8e89, 0x339494a7, 
	0x2d9b9bb6, 0x3c1e1e22, 0x15878792, 0xc9e9e920, 
	0x87cece49, 0xaa5555ff, 0x50282878, 0xa5dfdf7a, 
	0x038c8c8f, 0x59a1a1f8, 0x09898980, 0x1a0d0d17, 
	0x65bfbfda, 0xd7e6e631, 0x844242c6, 0xd06868b8, 
	0x824141c3, 0x299999b0, 0x5a2d2d77, 0x1e0f0f11, 
	0x7bb0b0cb, 0xa85454fc, 0x6dbbbbd6, 0x2c16163a
};

static const u32 TBLB1[256]={
	0xa5c66363, 0x84f87c7c, 0x99ee7777, 0x8df67b7b, 
	0x0dfff2f2, 0xbdd66b6b, 0xb1de6f6f, 0x5491c5c5, 
	0x50603030, 0x03020101, 0xa9ce6767, 0x7d562b2b, 
	0x19e7fefe, 0x62b5d7d7, 0xe64dabab, 0x9aec7676, 
	0x458fcaca, 0x9d1f8282, 0x4089c9c9, 0x87fa7d7d, 
	0x15effafa, 0xebb25959, 0xc98e4747, 0x0bfbf0f0, 
	0xec41adad, 0x67b3d4d4, 0xfd5fa2a2, 0xea45afaf, 
	0xbf239c9c, 0xf753a4a4, 0x96e47272, 0x5b9bc0c0, 
	0xc275b7b7, 0x1ce1fdfd, 0xae3d9393, 0x6a4c2626, 
	0x5a6c3636, 0x417e3f3f, 0x02f5f7f7, 0x4f83cccc, 
	0x5c683434, 0xf451a5a5, 0x34d1e5e5, 0x08f9f1f1, 
	0x93e27171, 0x73abd8d8, 0x53623131, 0x3f2a1515, 
	0x0c080404, 0x5295c7c7, 0x65462323, 0x5e9dc3c3, 
	0x28301818, 0xa1379696, 0x0f0a0505, 0xb52f9a9a, 
	0x090e0707, 0x36241212, 0x9b1b8080, 0x3ddfe2e2, 
	0x26cdebeb, 0x694e2727, 0xcd7fb2b2, 0x9fea7575, 
	0x1b120909, 0x9e1d8383, 0x74582c2c, 0x2e341a1a, 
	0x2d361b1b, 0xb2dc6e6e, 0xeeb45a5a, 0xfb5ba0a0, 
	0xf6a45252, 0x4d763b3b, 0x61b7d6d6, 0xce7db3b3, 
	0x7b522929, 0x3edde3e3, 0x715e2f2f, 0x97138484, 
	0xf5a65353, 0x68b9d1d1, 0x00000000, 0x2cc1eded, 
	0x60402020, 0x1fe3fcfc, 0xc879b1b1, 0xedb65b5b, 
	0xbed46a6a, 0x468dcbcb, 0xd967bebe, 0x4b723939, 
	0xde944a4a, 0xd4984c4c, 0xe8b05858, 0x4a85cfcf, 
	0x6bbbd0d0, 0x2ac5efef, 0xe54faaaa, 0x16edfbfb, 
	0xc5864343, 0xd79a4d4d, 0x55663333, 0x94118585, 
	0xcf8a4545, 0x10e9f9f9, 0x06040202, 0x81fe7f7f, 
	0xf0a05050, 0x44783c3c, 0xba259f9f, 0xe34ba8a8, 
	0xf3a25151, 0xfe5da3a3, 0xc0804040, 0x8a058f8f, 
	0xad3f9292, 0xbc219d9d, 0x48703838, 0x04f1f5f5, 
	0xdf63bcbc, 0xc177b6b6, 0x75afdada, 0x63422121, 
	0x30201010, 0x1ae5ffff, 0x0efdf3f3, 0x6dbfd2d2, 
	0x4c81cdcd, 0x14180c0c, 0x35261313, 0x2fc3ecec, 
	0xe1be5f5f, 0xa2359797, 0xcc884444, 0x392e1717, 
	0x5793c4c4, 0xf255a7a7, 0x82fc7e7e, 0x477a3d3d, 
	0xacc86464, 0xe7ba5d5d, 0x2b321919, 0x95e67373, 
	0xa0c06060, 0x98198181, 0xd19e4f4f, 0x7fa3dcdc, 
	0x66442222, 0x7e542a2a, 0xab3b9090, 0x830b8888, 
	0xca8c4646, 0x29c7eeee, 0xd36bb8b8, 0x3c281414, 
	0x79a7dede, 0xe2bc5e5e, 0x1d160b0b, 0x76addbdb, 
	0x3bdbe0e0, 0x56643232, 0x4e743a3a, 0x1e140a0a, 
	0xdb924949, 0x0a0c0606, 0x6c482424, 0xe4b85c5c, 
	0x5d9fc2c2, 0x6ebdd3d3, 0xef43acac, 0xa6c46262, 
	0xa8399191, 0xa4319595, 0x37d3e4e4, 0x8bf27979, 
	0x32d5e7e7, 0x438bc8c8, 0x596e3737, 0xb7da6d6d, 
	0x8c018d8d, 0x64b1d5d5, 0xd29c4e4e, 0xe049a9a9, 
	0xb4d86c6c, 0xfaac5656, 0x07f3f4f4, 0x25cfeaea, 
	0xafca6565, 0x8ef47a7a, 0xe947aeae, 0x18100808, 
	0xd56fbaba, 0x88f07878, 0x6f4a2525, 0x725c2e2e, 
	0x24381c1c, 0xf157a6a6, 0xc773b4b4, 0x5197c6c6, 
	0x23cbe8e8, 0x7ca1dddd, 0x9ce87474, 0x213e1f1f, 
	0xdd964b4b, 0xdc61bdbd, 0x860d8b8b, 0x850f8a8a, 
	0x90e07070, 0x427c3e3e, 0xc471b5b5, 0xaacc6666, 
	0xd8904848, 0x05060303, 0x01f7f6f6, 0x121c0e0e, 
	0xa3c26161, 0x5f6a3535, 0xf9ae5757, 0xd069b9b9, 
	0x91178686, 0x5899c1c1, 0x273a1d1d, 0xb9279e9e, 
	0x38d9e1e1, 0x13ebf8f8, 0xb32b9898, 0x33221111, 
	0xbbd26969, 0x70a9d9d9, 0x89078e8e, 0xa7339494, 
	0xb62d9b9b, 0x223c1e1e, 0x92158787, 0x20c9e9e9, 
	0x4987cece, 0xffaa5555, 0x78502828, 0x7aa5dfdf, 
	0x8f038c8c, 0xf859a1a1, 0x80098989, 0x171a0d0d, 
	0xda65bfbf, 0x31d7e6e6, 0xc6844242, 0xb8d06868, 
	0xc3824141, 0xb0299999, 0x775a2d2d, 0x111e0f0f, 
	0xcb7bb0b0, 0xfca85454, 0xd66dbbbb, 0x3a2c1616
};

static const u32 TBLB2[256]={
	0x63a5c663, 0x7c84f87c, 0x7799ee77, 0x7b8df67b, 
	0xf20dfff2, 0x6bbdd66b, 0x6fb1de6f, 0xc55491c5, 
	0x30506030, 0x01030201, 0x67a9ce67, 0x2b7d562b, 
	0xfe19e7fe, 0xd762b5d7, 0xabe64dab, 0x769aec76, 
	0xca458fca, 0x829d1f82, 0xc94089c9, 0x7d87fa7d, 
	0xfa15effa, 0x59ebb259, 0x47c98e47, 0xf00bfbf0, 
	0xadec41ad, 0xd467b3d4, 0xa2fd5fa2, 0xafea45af, 
	0x9cbf239c, 0xa4f753a4, 0x7296e472, 0xc05b9bc0, 
	0xb7c275b7, 0xfd1ce1fd, 0x93ae3d93, 0x266a4c26, 
	0x365a6c36, 0x3f417e3f, 0xf702f5f7, 0xcc4f83cc, 
	0x345c6834, 0xa5f451a5, 0xe534d1e5, 0xf108f9f1, 
	0x7193e271, 0xd873abd8, 0x31536231, 0x153f2a15, 
	0x040c0804, 0xc75295c7, 0x23654623, 0xc35e9dc3, 
	0x18283018, 0x96a13796, 0x050f0a05, 0x9ab52f9a, 
	0x07090e07, 0x12362412, 0x809b1b80, 0xe23ddfe2, 
	0xeb26cdeb, 0x27694e27, 0xb2cd7fb2, 0x759fea75, 
	0x091b1209, 0x839e1d83, 0x2c74582c, 0x1a2e341a, 
	0x1b2d361b, 0x6eb2dc6e, 0x5aeeb45a, 0xa0fb5ba0, 
	0x52f6a452, 0x3b4d763b, 0xd661b7d6, 0xb3ce7db3, 
	0x297b5229, 0xe33edde3, 0x2f715e2f, 0x84971384, 
	0x53f5a653, 0xd168b9d1, 0x00000000, 0xed2cc1ed, 
	0x20604020, 0xfc1fe3fc, 0xb1c879b1, 0x5bedb65b, 
	0x6abed46a, 0xcb468dcb, 0xbed967be, 0x394b7239, 
	0x4ade944a, 0x4cd4984c, 0x58e8b058, 0xcf4a85cf, 
	0xd06bbbd0, 0xef2ac5ef, 0xaae54faa, 0xfb16edfb, 
	0x43c58643, 0x4dd79a4d, 0x33556633, 0x85941185, 
	0x45cf8a45, 0xf910e9f9, 0x02060402, 0x7f81fe7f, 
	0x50f0a050, 0x3c44783c, 0x9fba259f, 0xa8e34ba8, 
	0x51f3a251, 0xa3fe5da3, 0x40c08040, 0x8f8a058f, 
	0x92ad3f92, 0x9dbc219d, 0x38487038, 0xf504f1f5, 
	0xbcdf63bc, 0xb6c177b6, 0xda75afda, 0x21634221, 
	0x10302010, 0xff1ae5ff, 0xf30efdf3, 0xd26dbfd2, 
	0xcd4c81cd, 0x0c14180c, 0x13352613, 0xec2fc3ec, 
	0x5fe1be5f, 0x97a23597, 0x44cc8844, 0x17392e17, 
	0xc45793c4, 0xa7f255a7, 0x7e82fc7e, 0x3d477a3d, 
	0x64acc864, 0x5de7ba5d, 0x192b3219, 0x7395e673, 
	0x60a0c060, 0x81981981, 0x4fd19e4f, 0xdc7fa3dc, 
	0x22664422, 0x2a7e542a, 0x90ab3b90, 0x88830b88, 
	0x46ca8c46, 0xee29c7ee, 0xb8d36bb8, 0x143c2814, 
	0xde79a7de, 0x5ee2bc5e, 0x0b1d160b, 0xdb76addb, 
	0xe03bdbe0, 0x32566432, 0x3a4e743a, 0x0a1e140a, 
	0x49db9249, 0x060a0c06, 0x246c4824, 0x5ce4b85c, 
	0xc25d9fc2, 0xd36ebdd3, 0xacef43ac, 0x62a6c462, 
	0x91a83991, 0x95a43195, 0xe437d3e4, 0x798bf279, 
	0xe732d5e7, 0xc8438bc8, 0x37596e37, 0x6db7da6d, 
	0x8d8c018d, 0xd564b1d5, 0x4ed29c4e, 0xa9e049a9, 
	0x6cb4d86c, 0x56faac56, 0xf407f3f4, 0xea25cfea, 
	0x65afca65, 0x7a8ef47a, 0xaee947ae, 0x08181008, 
	0xbad56fba, 0x7888f078, 0x256f4a25, 0x2e725c2e, 
	0x1c24381c, 0xa6f157a6, 0xb4c773b4, 0xc65197c6, 
	0xe823cbe8, 0xdd7ca1dd, 0x749ce874, 0x1f213e1f, 
	0x4bdd964b, 0xbddc61bd, 0x8b860d8b, 0x8a850f8a, 
	0x7090e070, 0x3e427c3e, 0xb5c471b5, 0x66aacc66, 
	0x48d89048, 0x03050603, 0xf601f7f6, 0x0e121c0e, 
	0x61a3c261, 0x355f6a35, 0x57f9ae57, 0xb9d069b9, 
	0x86911786, 0xc15899c1, 0x1d273a1d, 0x9eb9279e, 
	0xe138d9e1, 0xf813ebf8, 0x98b32b98, 0x11332211, 
	0x69bbd269, 0xd970a9d9, 0x8e89078e, 0x94a73394, 
	0x9bb62d9b, 0x1e223c1e, 0x87921587, 0xe920c9e9, 
	0xce4987ce, 0x55ffaa55, 0x28785028, 0xdf7aa5df, 
	0x8c8f038c, 0xa1f859a1, 0x89800989, 0x0d171a0d, 
	0xbfda65bf, 0xe631d7e6, 0x42c68442, 0x68b8d068, 
	0x41c38241, 0x99b02999, 0x2d775a2d, 0x0f111e0f, 
	0xb0cb7bb0, 0x54fca854, 0xbbd66dbb, 0x163a2c16
};

static const u32 TBLB3[256]={
	0x6363a5c6, 0x7c7c84f8, 0x777799ee, 0x7b7b8df6, 
	0xf2f20dff, 0x6b6bbdd6, 0x6f6fb1de, 0xc5c55491, 
	0x30305060, 0x01010302, 0x6767a9ce, 0x2b2b7d56, 
	0xfefe19e7, 0xd7d762b5, 0xababe64d, 0x76769aec, 
	0xcaca458f, 0x82829d1f, 0xc9c94089, 0x7d7d87fa, 
	0xfafa15ef, 0x5959ebb2, 0x4747c98e, 0xf0f00bfb, 
	0xadadec41, 0xd4d467b3, 0xa2a2fd5f, 0xafafea45, 
	0x9c9cbf23, 0xa4a4f753, 0x727296e4, 0xc0c05b9b, 
	0xb7b7c275, 0xfdfd1ce1, 0x9393ae3d, 0x26266a4c, 
	0x36365a6c, 0x3f3f417e, 0xf7f702f5, 0xcccc4f83, 
	0x34345c68, 0xa5a5f451, 0xe5e534d1, 0xf1f108f9, 
	0x717193e2, 0xd8d873ab, 0x31315362, 0x15153f2a, 
	0x04040c08, 0xc7c75295, 0x23236546, 0xc3c35e9d, 
	0x18182830, 0x9696a137, 0x05050f0a, 0x9a9ab52f, 
	0x0707090e, 0x12123624, 0x80809b1b, 0xe2e23ddf, 
	0xebeb26cd, 0x2727694e, 0xb2b2cd7f, 0x75759fea, 
	0x09091b12, 0x83839e1d, 0x2c2c7458, 0x1a1a2e34, 
	0x1b1b2d36, 0x6e6eb2dc, 0x5a5aeeb4, 0xa0a0fb5b, 
	0x5252f6a4, 0x3b3b4d76, 0xd6d661b7, 0xb3b3ce7d, 
	0x29297b52, 0xe3e33edd, 0x2f2f715e, 0x84849713, 
	0x5353f5a6, 0xd1d168b9, 0x00000000, 0xeded2cc1, 
	0x20206040, 0xfcfc1fe3, 0xb1b1c879, 0x5b5bedb6, 
	0x6a6abed4, 0xcbcb468d, 0xbebed967, 0x39394b72, 
	0x4a4ade94, 0x4c4cd498, 0x5858e8b0, 0xcfcf4a85, 
	0xd0d06bbb, 0xefef2ac5, 0xaaaae54f, 0xfbfb16ed, 
	0x4343c586, 0x4d4dd79a, 0x33335566, 0x85859411, 
	0x4545cf8a, 0xf9f910e9, 0x02020604, 0x7f7f81fe, 
	0x5050f0a0, 0x3c3c4478, 0x9f9fba25, 0xa8a8e34b, 
	0x5151f3a2, 0xa3a3fe5d, 0x4040c080, 0x8f8f8a05, 
	0x9292ad3f, 0x9d9dbc21, 0x38384870, 0xf5f504f1, 
	0xbcbcdf63, 0xb6b6c177, 0xdada75af, 0x21216342, 
	0x10103020, 0xffff1ae5, 0xf3f30efd, 0xd2d26dbf, 
	0xcdcd4c81, 0x0c0c1418, 0x13133526, 0xecec2fc3, 
	0x5f5fe1be, 0x9797a235, 0x4444cc88, 0x1717392e, 
	0xc4c45793, 0xa7a7f255, 0x7e7e82fc, 0x3d3d477a, 
	0x6464acc8, 0x5d5de7ba, 0x19192b32, 0x737395e6, 
	0x6060a0c0, 0x81819819, 0x4f4fd19e, 0xdcdc7fa3, 
	0x22226644, 0x2a2a7e54, 0x9090ab3b, 0x8888830b, 
	0x4646ca8c, 0xeeee29c7, 0xb8b8d36b, 0x14143c28, 
	0xdede79a7, 0x5e5ee2bc, 0x0b0b1d16, 0xdbdb76ad, 
	0xe0e03bdb, 0x32325664, 0x3a3a4e74, 0x0a0a1e14, 
	0x4949db92, 0x06060a0c, 0x24246c48, 0x5c5ce4b8, 
	0xc2c25d9f, 0xd3d36ebd, 0xacacef43, 0x6262a6c4, 
	0x9191a839, 0x9595a431, 0xe4e437d3, 0x79798bf2, 
	0xe7e732d5, 0xc8c8438b, 0x3737596e, 0x6d6db7da, 
	0x8d8d8c01, 0xd5d564b1, 0x4e4ed29c, 0xa9a9e049, 
	0x6c6cb4d8, 0x5656faac, 0xf4f407f3, 0xeaea25cf, 
	0x6565afca, 0x7a7a8ef4, 0xaeaee947, 0x08081810, 
	0xbabad56f, 0x787888f0, 0x25256f4a, 0x2e2e725c, 
	0x1c1c2438, 0xa6a6f157, 0xb4b4c773, 0xc6c65197, 
	0xe8e823cb, 0xdddd7ca1, 0x74749ce8, 0x1f1f213e, 
	0x4b4bdd96, 0xbdbddc61, 0x8b8b860d, 0x8a8a850f, 
	0x707090e0, 0x3e3e427c, 0xb5b5c471, 0x6666aacc, 
	0x4848d890, 0x03030506, 0xf6f601f7, 0x0e0e121c, 
	0x6161a3c2, 0x35355f6a, 0x5757f9ae, 0xb9b9d069, 
	0x86869117, 0xc1c15899, 0x1d1d273a, 0x9e9eb927, 
	0xe1e138d9, 0xf8f813eb, 0x9898b32b, 0x11113322, 
	0x6969bbd2, 0xd9d970a9, 0x8e8e8907, 0x9494a733, 
	0x9b9bb62d, 0x1e1e223c, 0x87879215, 0xe9e920c9, 
	0xcece4987, 0x5555ffaa, 0x28287850, 0xdfdf7aa5, 
	0x8c8c8f03, 0xa1a1f859, 0x89898009, 0x0d0d171a, 
	0xbfbfda65, 0xe6e631d7, 0x4242c684, 0x6868b8d0, 
	0x4141c382, 0x9999b029, 0x2d2d775a, 0x0f0f111e, 
	0xb0b0cb7b, 0x5454fca8, 0xbbbbd66d, 0x16163a2c
};

static const u64 TBLC0[256] = {
	0x6397633556f3c663LL, 0x7ceb7ccdb135f87cLL, 0x77c77795e27bee77LL, 0x7bf77bf58e03f67bLL,
	0xf2e5f2d1232efff2LL, 0x6bb76b751ea3d66bLL, 0x6fa76f553a8bde6fLL, 0xc539c572b7e391c5LL,
	0x30c0309babfb6030LL, 0x01040108090a0201LL, 0x6787671572dbce67LL, 0x2bac2b436815562bLL,
	0xfed5feb14f56e7feLL, 0xd771d7e23557b5d7LL, 0xab9aab2f84624dabLL, 0x76c3769deb71ec76LL,
	0xca05ca0ac0858fcaLL, 0x823e827cfe631f82LL, 0xc909c912db9b89c9LL, 0x7def7dc5b83ffa7dLL,
	0xfac5fa916b7eeffaLL, 0x597f59fea74cb259LL, 0x4707470e49808e47LL, 0xf0edf0c1313afbf0LL,
	0xad82ad1fb25e41adLL, 0xd47dd4fa2e49b3d4LL, 0xa2bea267c5385fa2LL, 0xaf8aaf0fa04a45afLL,
	0x9c469c8c10af239cLL, 0xa4a6a457f30453a4LL, 0x72d372bdcf59e472LL, 0xc02dc05a9ac19bc0LL,
	0xb7eab7cf78ba75b7LL, 0xfdd9fda95448e1fdLL, 0x937a93f467c93d93LL, 0x2698262b0d674c26LL,
	0x36d836ab9dc76c36LL, 0x3ffc3fe3dc9d7e3fLL, 0xf7f1f7f90e0cf5f7LL, 0xcc1dcc3af6b983ccLL,
	0x34d034bb8fd36834LL, 0xa5a2a55ffa0e51a5LL, 0xe5b9e5698cb8d1e5LL, 0xf1e9f1c93830f9f1LL,
	0x71df71a5d447e271LL, 0xd84dd89a4231abd8LL, 0x31c43193a2f16231LL, 0x155415a8bd822a15LL,
	0x0410042024280804LL, 0xc731c762a5f795c7LL, 0x238c230320454623LL, 0xc321c34281df9dc3LL,
	0x186018c0d8f03018LL, 0x966e96dc4aeb3796LL, 0x051405282d220a05LL, 0x9a5e9abc26932f9aLL,
	0x071c07383f360e07LL, 0x1248129082b42412LL, 0x8036806cec771b80LL, 0xe2a5e251b38edfe2LL,
	0xeb81eb19f2d4cdebLL, 0x279c2723046d4e27LL, 0xb2feb2e755987fb2LL, 0x75cf7585f06fea75LL,
	0x09240948415a1209LL, 0x833a8374f7691d83LL, 0x2cb02c7b5723582cLL, 0x1a681ad0cae4341aLL,
	0x1b6c1bd8c3ee361bLL, 0x6ea36e5d3381dc6eLL, 0x5a735ae6bc52b45aLL, 0xa0b6a077d72c5ba0LL,
	0x525352a6f402a452LL, 0x3bec3bc3f8b5763bLL, 0xd675d6ea3c5db7d6LL, 0xb3fab3ef5c927db3LL,
	0x29a429537a015229LL, 0xe3a1e359ba84dde3LL, 0x2fbc2f634c3d5e2fLL, 0x8426844cc85f1384LL,
	0x535753aefd08a653LL, 0xd169d1d2036bb9d1LL, 0x0000000000000000LL, 0xed99ed29c4e8c1edLL,
	0x2080201b3b5b4020LL, 0xfcddfca15d42e3fcLL, 0xb1f2b1ff4e8679b1LL, 0x5b775beeb558b65bLL,
	0x6ab36a7d17a9d46aLL, 0xcb01cb02c98f8dcbLL, 0xbecebe8739e067beLL, 0x39e439d3eaa17239LL,
	0x4a334a662cf2944aLL, 0x4c2b4c561ace984cLL, 0x587b58f6ae46b058LL, 0xcf11cf22eda785cfLL,
	0xd06dd0da0a61bbd0LL, 0xef91ef39d6fcc5efLL, 0xaa9eaa278d684faaLL, 0xfbc1fb996274edfbLL,
	0x4317432e6da88643LL, 0x4d2f4d5e13c49a4dLL, 0x33cc3383b0e56633LL, 0x85228544c1551185LL,
	0x450f451e5b948a45LL, 0xf9c9f9897060e9f9LL, 0x0208021012140402LL, 0x7fe77fd5aa2bfe7fLL,
	0x505b50b6e616a050LL, 0x3cf03cfbc783783cLL, 0x9f4a9f940bb1259fLL, 0xa896a8379f7c4ba8LL,
	0x515f51beef1ca251LL, 0xa3baa36fcc325da3LL, 0x401b403676b68040LL, 0x8f0a8f149b11058fLL,
	0x927e92fc6ec33f92LL, 0x9d429d8419a5219dLL, 0x38e038dbe3ab7038LL, 0xf5f9f5e91c18f1f5LL,
	0xbcc6bc972bf463bcLL, 0xb6eeb6c771b077b6LL, 0xda45da8a5025afdaLL, 0x2184211332514221LL,
	0x1040108090a02010LL, 0xffd1ffb9465ce5ffLL, 0xf3e1f3d92a24fdf3LL, 0xd265d2ca1875bfd2LL,
	0xcd19cd32ffb381cdLL, 0x0c300c606c78180cLL, 0x134c13988bbe2613LL, 0xec9dec21cde2c3ecLL,
	0x5f675fce9170be5fLL, 0x976a97d443e13597LL, 0x440b4416529e8844LL, 0x175c17b8af962e17LL,
	0xc43dc47abee993c4LL, 0xa7aaa74fe81a55a7LL, 0x7ee37edda321fc7eLL, 0x3df43df3ce897a3dLL,
	0x648b640d69c5c864LL, 0x5d6f5dde8364ba5dLL, 0x196419c8d1fa3219LL, 0x73d773b5c653e673LL,
	0x609b602d4dedc060LL, 0x81328164e57d1981LL, 0x4f274f4e01d09e4fLL, 0xdc5ddcba6619a3dcLL,
	0x2288220b294f4422LL, 0x2aa82a4b611f542aLL, 0x907690ec7cd73b90LL, 0x8816882ca4270b88LL,
	0x46034606408a8c46LL, 0xee95ee31dff6c7eeLL, 0xb8d6b8b70fdc6bb8LL, 0x145014a0b4882814LL,
	0xde55deaa740da7deLL, 0x5e635ec6987abc5eLL, 0x0b2c0b58534e160bLL, 0xdb41db82592faddbLL,
	0xe0ade041a19adbe0LL, 0x32c8328bb9ef6432LL, 0x3ae83acbf1bf743aLL, 0x0a280a505a44140aLL,
	0x493f497e37ec9249LL, 0x06180630363c0c06LL, 0x2490243b1f734824LL, 0x5c6b5cd68a6eb85cLL,
	0xc225c24a88d59fc2LL, 0xd361d3c2117fbdd3LL, 0xac86ac17bb5443acLL, 0x6293623d5ff9c462LL,
	0x917291e475dd3991LL, 0x956295c451f53195LL, 0xe4bde46185b2d3e4LL, 0x79ff79e59c17f279LL,
	0xe7b1e7799eacd5e7LL, 0xc80dc81ad2918bc8LL, 0x37dc37a394cd6e37LL, 0x6daf6d45289fda6dLL,
	0x8d028d048905018dLL, 0xd579d5f22743b1d5LL, 0x4e234e4608da9c4eLL, 0xa992a93f967649a9LL,
	0x6cab6c4d2195d86cLL, 0x56435686d02aac56LL, 0xf4fdf4e11512f3f4LL, 0xea85ea11fbdecfeaLL,
	0x658f650560cfca65LL, 0x7af37afd8709f47aLL, 0xae8eae07a94047aeLL, 0x0820084048501008LL,
	0xbadebaa71dc86fbaLL, 0x78fb78ed951df078LL, 0x2594253316794a25LL, 0x2eb82e6b45375c2eLL,
	0x1c701ce0fcd8381cLL, 0xa6aea647e11057a6LL, 0xb4e6b4d763a473b4LL, 0xc635c66aacfd97c6LL,
	0xe88de801e9cacbe8LL, 0xdd59ddb26f13a1ddLL, 0x74cb748df965e874LL, 0x1f7c1ff8e7c63e1fLL,
	0x4b374b6e25f8964bLL, 0xbdc2bd9f22fe61bdLL, 0x8b1a8b34bf390d8bLL, 0x8a1e8a3cb6330f8aLL,
	0x70db70addd4de070LL, 0x3ef83eebd5977c3eLL, 0xb5e2b5df6aae71b5LL, 0x6683661d7bd1cc66LL,
	0x483b48763ee69048LL, 0x030c03181b1e0603LL, 0xf6f5f6f10706f7f6LL, 0x0e380e707e6c1c0eLL,
	0x619f612544e7c261LL, 0x35d435b386d96a35LL, 0x5747578ed920ae57LL, 0xb9d2b9bf06d669b9LL,
	0x862e865cda4b1786LL, 0xc129c15293cb99c1LL, 0x1d741de8f5d23a1dLL, 0x9e4e9e9c02bb279eLL,
	0xe1a9e149a890d9e1LL, 0xf8cdf881796aebf8LL, 0x985698ac34872b98LL, 0x1144118899aa2211LL,
	0x69bf69650cb7d269LL, 0xd949d9924b3ba9d9LL, 0x8e0e8e1c921b078eLL, 0x946694cc58ff3394LL,
	0x9b5a9bb42f992d9bLL, 0x1e781ef0eecc3c1eLL, 0x872a8754d3411587LL, 0xe989e909e0c0c9e9LL,
	0xce15ce2ae4ad87ceLL, 0x554f559ecb34aa55LL, 0x28a0285b730b5028LL, 0xdf51dfa27d07a5dfLL,
	0x8c068c0c800f038cLL, 0xa1b2a17fde2659a1LL, 0x89128924ad2d0989LL, 0x0d340d6865721a0dLL,
	0xbfcabf8f30ea65bfLL, 0xe6b5e67197a6d7e6LL, 0x4213422664a28442LL, 0x68bb686d05bdd068LL,
	0x411f413e7fbc8241LL, 0x995299a43d8d2999LL, 0x2db42d735e295a2dLL, 0x0f3c0f7877661e0fLL,
	0xb0f6b0f7478c7bb0LL, 0x544b5496c23ea854LL, 0xbbdabbaf14c26dbbLL, 0x165816b0a69c2c16LL
};

static const u64 TBLC1[256] = {
	0x636397633556f3c6LL, 0x7c7ceb7ccdb135f8LL, 0x7777c77795e27beeLL, 0x7b7bf77bf58e03f6LL,
	0xf2f2e5f2d1232effLL, 0x6b6bb76b751ea3d6LL, 0x6f6fa76f553a8bdeLL, 0xc5c539c572b7e391LL,
	0x3030c0309babfb60LL, 0x0101040108090a02LL, 0x676787671572dbceLL, 0x2b2bac2b43681556LL,
	0xfefed5feb14f56e7LL, 0xd7d771d7e23557b5LL, 0xabab9aab2f84624dLL, 0x7676c3769deb71ecLL,
	0xcaca05ca0ac0858fLL, 0x82823e827cfe631fLL, 0xc9c909c912db9b89LL, 0x7d7def7dc5b83ffaLL,
	0xfafac5fa916b7eefLL, 0x59597f59fea74cb2LL, 0x474707470e49808eLL, 0xf0f0edf0c1313afbLL,
	0xadad82ad1fb25e41LL, 0xd4d47dd4fa2e49b3LL, 0xa2a2bea267c5385fLL, 0xafaf8aaf0fa04a45LL,
	0x9c9c469c8c10af23LL, 0xa4a4a6a457f30453LL, 0x7272d372bdcf59e4LL, 0xc0c02dc05a9ac19bLL,
	0xb7b7eab7cf78ba75LL, 0xfdfdd9fda95448e1LL, 0x93937a93f467c93dLL, 0x262698262b0d674cLL,
	0x3636d836ab9dc76cLL, 0x3f3ffc3fe3dc9d7eLL, 0xf7f7f1f7f90e0cf5LL, 0xcccc1dcc3af6b983LL,
	0x3434d034bb8fd368LL, 0xa5a5a2a55ffa0e51LL, 0xe5e5b9e5698cb8d1LL, 0xf1f1e9f1c93830f9LL,
	0x7171df71a5d447e2LL, 0xd8d84dd89a4231abLL, 0x3131c43193a2f162LL, 0x15155415a8bd822aLL,
	0x0404100420242808LL, 0xc7c731c762a5f795LL, 0x23238c2303204546LL, 0xc3c321c34281df9dLL,
	0x18186018c0d8f030LL, 0x96966e96dc4aeb37LL, 0x05051405282d220aLL, 0x9a9a5e9abc26932fLL,
	0x07071c07383f360eLL, 0x121248129082b424LL, 0x808036806cec771bLL, 0xe2e2a5e251b38edfLL,
	0xebeb81eb19f2d4cdLL, 0x27279c2723046d4eLL, 0xb2b2feb2e755987fLL, 0x7575cf7585f06feaLL,
	0x0909240948415a12LL, 0x83833a8374f7691dLL, 0x2c2cb02c7b572358LL, 0x1a1a681ad0cae434LL,
	0x1b1b6c1bd8c3ee36LL, 0x6e6ea36e5d3381dcLL, 0x5a5a735ae6bc52b4LL, 0xa0a0b6a077d72c5bLL,
	0x52525352a6f402a4LL, 0x3b3bec3bc3f8b576LL, 0xd6d675d6ea3c5db7LL, 0xb3b3fab3ef5c927dLL,
	0x2929a429537a0152LL, 0xe3e3a1e359ba84ddLL, 0x2f2fbc2f634c3d5eLL, 0x848426844cc85f13LL,
	0x53535753aefd08a6LL, 0xd1d169d1d2036bb9LL, 0x0000000000000000LL, 0xeded99ed29c4e8c1LL,
	0x202080201b3b5b40LL, 0xfcfcddfca15d42e3LL, 0xb1b1f2b1ff4e8679LL, 0x5b5b775beeb558b6LL,
	0x6a6ab36a7d17a9d4LL, 0xcbcb01cb02c98f8dLL, 0xbebecebe8739e067LL, 0x3939e439d3eaa172LL,
	0x4a4a334a662cf294LL, 0x4c4c2b4c561ace98LL, 0x58587b58f6ae46b0LL, 0xcfcf11cf22eda785LL,
	0xd0d06dd0da0a61bbLL, 0xefef91ef39d6fcc5LL, 0xaaaa9eaa278d684fLL, 0xfbfbc1fb996274edLL,
	0x434317432e6da886LL, 0x4d4d2f4d5e13c49aLL, 0x3333cc3383b0e566LL, 0x8585228544c15511LL,
	0x45450f451e5b948aLL, 0xf9f9c9f9897060e9LL, 0x0202080210121404LL, 0x7f7fe77fd5aa2bfeLL,
	0x50505b50b6e616a0LL, 0x3c3cf03cfbc78378LL, 0x9f9f4a9f940bb125LL, 0xa8a896a8379f7c4bLL,
	0x51515f51beef1ca2LL, 0xa3a3baa36fcc325dLL, 0x40401b403676b680LL, 0x8f8f0a8f149b1105LL,
	0x92927e92fc6ec33fLL, 0x9d9d429d8419a521LL, 0x3838e038dbe3ab70LL, 0xf5f5f9f5e91c18f1LL,
	0xbcbcc6bc972bf463LL, 0xb6b6eeb6c771b077LL, 0xdada45da8a5025afLL, 0x2121842113325142LL,
	0x101040108090a020LL, 0xffffd1ffb9465ce5LL, 0xf3f3e1f3d92a24fdLL, 0xd2d265d2ca1875bfLL,
	0xcdcd19cd32ffb381LL, 0x0c0c300c606c7818LL, 0x13134c13988bbe26LL, 0xecec9dec21cde2c3LL,
	0x5f5f675fce9170beLL, 0x97976a97d443e135LL, 0x44440b4416529e88LL, 0x17175c17b8af962eLL,
	0xc4c43dc47abee993LL, 0xa7a7aaa74fe81a55LL, 0x7e7ee37edda321fcLL, 0x3d3df43df3ce897aLL,
	0x64648b640d69c5c8LL, 0x5d5d6f5dde8364baLL, 0x19196419c8d1fa32LL, 0x7373d773b5c653e6LL,
	0x60609b602d4dedc0LL, 0x8181328164e57d19LL, 0x4f4f274f4e01d09eLL, 0xdcdc5ddcba6619a3LL,
	0x222288220b294f44LL, 0x2a2aa82a4b611f54LL, 0x90907690ec7cd73bLL, 0x888816882ca4270bLL,
	0x4646034606408a8cLL, 0xeeee95ee31dff6c7LL, 0xb8b8d6b8b70fdc6bLL, 0x14145014a0b48828LL,
	0xdede55deaa740da7LL, 0x5e5e635ec6987abcLL, 0x0b0b2c0b58534e16LL, 0xdbdb41db82592fadLL,
	0xe0e0ade041a19adbLL, 0x3232c8328bb9ef64LL, 0x3a3ae83acbf1bf74LL, 0x0a0a280a505a4414LL,
	0x49493f497e37ec92LL, 0x0606180630363c0cLL, 0x242490243b1f7348LL, 0x5c5c6b5cd68a6eb8LL,
	0xc2c225c24a88d59fLL, 0xd3d361d3c2117fbdLL, 0xacac86ac17bb5443LL, 0x626293623d5ff9c4LL,
	0x91917291e475dd39LL, 0x95956295c451f531LL, 0xe4e4bde46185b2d3LL, 0x7979ff79e59c17f2LL,
	0xe7e7b1e7799eacd5LL, 0xc8c80dc81ad2918bLL, 0x3737dc37a394cd6eLL, 0x6d6daf6d45289fdaLL,
	0x8d8d028d04890501LL, 0xd5d579d5f22743b1LL, 0x4e4e234e4608da9cLL, 0xa9a992a93f967649LL,
	0x6c6cab6c4d2195d8LL, 0x5656435686d02aacLL, 0xf4f4fdf4e11512f3LL, 0xeaea85ea11fbdecfLL,
	0x65658f650560cfcaLL, 0x7a7af37afd8709f4LL, 0xaeae8eae07a94047LL, 0x0808200840485010LL,
	0xbabadebaa71dc86fLL, 0x7878fb78ed951df0LL, 0x252594253316794aLL, 0x2e2eb82e6b45375cLL,
	0x1c1c701ce0fcd838LL, 0xa6a6aea647e11057LL, 0xb4b4e6b4d763a473LL, 0xc6c635c66aacfd97LL,
	0xe8e88de801e9cacbLL, 0xdddd59ddb26f13a1LL, 0x7474cb748df965e8LL, 0x1f1f7c1ff8e7c63eLL,
	0x4b4b374b6e25f896LL, 0xbdbdc2bd9f22fe61LL, 0x8b8b1a8b34bf390dLL, 0x8a8a1e8a3cb6330fLL,
	0x7070db70addd4de0LL, 0x3e3ef83eebd5977cLL, 0xb5b5e2b5df6aae71LL, 0x666683661d7bd1ccLL,
	0x48483b48763ee690LL, 0x03030c03181b1e06LL, 0xf6f6f5f6f10706f7LL, 0x0e0e380e707e6c1cLL,
	0x61619f612544e7c2LL, 0x3535d435b386d96aLL, 0x575747578ed920aeLL, 0xb9b9d2b9bf06d669LL,
	0x86862e865cda4b17LL, 0xc1c129c15293cb99LL, 0x1d1d741de8f5d23aLL, 0x9e9e4e9e9c02bb27LL,
	0xe1e1a9e149a890d9LL, 0xf8f8cdf881796aebLL, 0x98985698ac34872bLL, 0x111144118899aa22LL,
	0x6969bf69650cb7d2LL, 0xd9d949d9924b3ba9LL, 0x8e8e0e8e1c921b07LL, 0x94946694cc58ff33LL,
	0x9b9b5a9bb42f992dLL, 0x1e1e781ef0eecc3cLL, 0x87872a8754d34115LL, 0xe9e989e909e0c0c9LL,
	0xcece15ce2ae4ad87LL, 0x55554f559ecb34aaLL, 0x2828a0285b730b50LL, 0xdfdf51dfa27d07a5LL,
	0x8c8c068c0c800f03LL, 0xa1a1b2a17fde2659LL, 0x8989128924ad2d09LL, 0x0d0d340d6865721aLL,
	0xbfbfcabf8f30ea65LL, 0xe6e6b5e67197a6d7LL, 0x424213422664a284LL, 0x6868bb686d05bdd0LL,
	0x41411f413e7fbc82LL, 0x99995299a43d8d29LL, 0x2d2db42d735e295aLL, 0x0f0f3c0f7877661eLL,
	0xb0b0f6b0f7478c7bLL, 0x54544b5496c23ea8LL, 0xbbbbdabbaf14c26dLL, 0x16165816b0a69c2cLL
};

static const u64 TBLC2[256] = {
	0xc6636397633556f3LL, 0xf87c7ceb7ccdb135LL, 0xee7777c77795e27bLL, 0xf67b7bf77bf58e03LL,
	0xfff2f2e5f2d1232eLL, 0xd66b6bb76b751ea3LL, 0xde6f6fa76f553a8bLL, 0x91c5c539c572b7e3LL,
	0x603030c0309babfbLL, 0x020101040108090aLL, 0xce676787671572dbLL, 0x562b2bac2b436815LL,
	0xe7fefed5feb14f56LL, 0xb5d7d771d7e23557LL, 0x4dabab9aab2f8462LL, 0xec7676c3769deb71LL,
	0x8fcaca05ca0ac085LL, 0x1f82823e827cfe63LL, 0x89c9c909c912db9bLL, 0xfa7d7def7dc5b83fLL,
	0xeffafac5fa916b7eLL, 0xb259597f59fea74cLL, 0x8e474707470e4980LL, 0xfbf0f0edf0c1313aLL,
	0x41adad82ad1fb25eLL, 0xb3d4d47dd4fa2e49LL, 0x5fa2a2bea267c538LL, 0x45afaf8aaf0fa04aLL,
	0x239c9c469c8c10afLL, 0x53a4a4a6a457f304LL, 0xe47272d372bdcf59LL, 0x9bc0c02dc05a9ac1LL,
	0x75b7b7eab7cf78baLL, 0xe1fdfdd9fda95448LL, 0x3d93937a93f467c9LL, 0x4c262698262b0d67LL,
	0x6c3636d836ab9dc7LL, 0x7e3f3ffc3fe3dc9dLL, 0xf5f7f7f1f7f90e0cLL, 0x83cccc1dcc3af6b9LL,
	0x683434d034bb8fd3LL, 0x51a5a5a2a55ffa0eLL, 0xd1e5e5b9e5698cb8LL, 0xf9f1f1e9f1c93830LL,
	0xe27171df71a5d447LL, 0xabd8d84dd89a4231LL, 0x623131c43193a2f1LL, 0x2a15155415a8bd82LL,
	0x0804041004202428LL, 0x95c7c731c762a5f7LL, 0x4623238c23032045LL, 0x9dc3c321c34281dfLL,
	0x3018186018c0d8f0LL, 0x3796966e96dc4aebLL, 0x0a05051405282d22LL, 0x2f9a9a5e9abc2693LL,
	0x0e07071c07383f36LL, 0x24121248129082b4LL, 0x1b808036806cec77LL, 0xdfe2e2a5e251b38eLL,
	0xcdebeb81eb19f2d4LL, 0x4e27279c2723046dLL, 0x7fb2b2feb2e75598LL, 0xea7575cf7585f06fLL,
	0x120909240948415aLL, 0x1d83833a8374f769LL, 0x582c2cb02c7b5723LL, 0x341a1a681ad0cae4LL,
	0x361b1b6c1bd8c3eeLL, 0xdc6e6ea36e5d3381LL, 0xb45a5a735ae6bc52LL, 0x5ba0a0b6a077d72cLL,
	0xa452525352a6f402LL, 0x763b3bec3bc3f8b5LL, 0xb7d6d675d6ea3c5dLL, 0x7db3b3fab3ef5c92LL,
	0x522929a429537a01LL, 0xdde3e3a1e359ba84LL, 0x5e2f2fbc2f634c3dLL, 0x13848426844cc85fLL,
	0xa653535753aefd08LL, 0xb9d1d169d1d2036bLL, 0x0000000000000000LL, 0xc1eded99ed29c4e8LL,
	0x40202080201b3b5bLL, 0xe3fcfcddfca15d42LL, 0x79b1b1f2b1ff4e86LL, 0xb65b5b775beeb558LL,
	0xd46a6ab36a7d17a9LL, 0x8dcbcb01cb02c98fLL, 0x67bebecebe8739e0LL, 0x723939e439d3eaa1LL,
	0x944a4a334a662cf2LL, 0x984c4c2b4c561aceLL, 0xb058587b58f6ae46LL, 0x85cfcf11cf22eda7LL,
	0xbbd0d06dd0da0a61LL, 0xc5efef91ef39d6fcLL, 0x4faaaa9eaa278d68LL, 0xedfbfbc1fb996274LL,
	0x86434317432e6da8LL, 0x9a4d4d2f4d5e13c4LL, 0x663333cc3383b0e5LL, 0x118585228544c155LL,
	0x8a45450f451e5b94LL, 0xe9f9f9c9f9897060LL, 0x0402020802101214LL, 0xfe7f7fe77fd5aa2bLL,
	0xa050505b50b6e616LL, 0x783c3cf03cfbc783LL, 0x259f9f4a9f940bb1LL, 0x4ba8a896a8379f7cLL,
	0xa251515f51beef1cLL, 0x5da3a3baa36fcc32LL, 0x8040401b403676b6LL, 0x058f8f0a8f149b11LL,
	0x3f92927e92fc6ec3LL, 0x219d9d429d8419a5LL, 0x703838e038dbe3abLL, 0xf1f5f5f9f5e91c18LL,
	0x63bcbcc6bc972bf4LL, 0x77b6b6eeb6c771b0LL, 0xafdada45da8a5025LL, 0x4221218421133251LL,
	0x20101040108090a0LL, 0xe5ffffd1ffb9465cLL, 0xfdf3f3e1f3d92a24LL, 0xbfd2d265d2ca1875LL,
	0x81cdcd19cd32ffb3LL, 0x180c0c300c606c78LL, 0x2613134c13988bbeLL, 0xc3ecec9dec21cde2LL,
	0xbe5f5f675fce9170LL, 0x3597976a97d443e1LL, 0x8844440b4416529eLL, 0x2e17175c17b8af96LL,
	0x93c4c43dc47abee9LL, 0x55a7a7aaa74fe81aLL, 0xfc7e7ee37edda321LL, 0x7a3d3df43df3ce89LL,
	0xc864648b640d69c5LL, 0xba5d5d6f5dde8364LL, 0x3219196419c8d1faLL, 0xe67373d773b5c653LL,
	0xc060609b602d4dedLL, 0x198181328164e57dLL, 0x9e4f4f274f4e01d0LL, 0xa3dcdc5ddcba6619LL,
	0x44222288220b294fLL, 0x542a2aa82a4b611fLL, 0x3b90907690ec7cd7LL, 0x0b888816882ca427LL,
	0x8c4646034606408aLL, 0xc7eeee95ee31dff6LL, 0x6bb8b8d6b8b70fdcLL, 0x2814145014a0b488LL,
	0xa7dede55deaa740dLL, 0xbc5e5e635ec6987aLL, 0x160b0b2c0b58534eLL, 0xaddbdb41db82592fLL,
	0xdbe0e0ade041a19aLL, 0x643232c8328bb9efLL, 0x743a3ae83acbf1bfLL, 0x140a0a280a505a44LL,
	0x9249493f497e37ecLL, 0x0c0606180630363cLL, 0x48242490243b1f73LL, 0xb85c5c6b5cd68a6eLL,
	0x9fc2c225c24a88d5LL, 0xbdd3d361d3c2117fLL, 0x43acac86ac17bb54LL, 0xc4626293623d5ff9LL,
	0x3991917291e475ddLL, 0x3195956295c451f5LL, 0xd3e4e4bde46185b2LL, 0xf27979ff79e59c17LL,
	0xd5e7e7b1e7799eacLL, 0x8bc8c80dc81ad291LL, 0x6e3737dc37a394cdLL, 0xda6d6daf6d45289fLL,
	0x018d8d028d048905LL, 0xb1d5d579d5f22743LL, 0x9c4e4e234e4608daLL, 0x49a9a992a93f9676LL,
	0xd86c6cab6c4d2195LL, 0xac5656435686d02aLL, 0xf3f4f4fdf4e11512LL, 0xcfeaea85ea11fbdeLL,
	0xca65658f650560cfLL, 0xf47a7af37afd8709LL, 0x47aeae8eae07a940LL, 0x1008082008404850LL,
	0x6fbabadebaa71dc8LL, 0xf07878fb78ed951dLL, 0x4a25259425331679LL, 0x5c2e2eb82e6b4537LL,
	0x381c1c701ce0fcd8LL, 0x57a6a6aea647e110LL, 0x73b4b4e6b4d763a4LL, 0x97c6c635c66aacfdLL,
	0xcbe8e88de801e9caLL, 0xa1dddd59ddb26f13LL, 0xe87474cb748df965LL, 0x3e1f1f7c1ff8e7c6LL,
	0x964b4b374b6e25f8LL, 0x61bdbdc2bd9f22feLL, 0x0d8b8b1a8b34bf39LL, 0x0f8a8a1e8a3cb633LL,
	0xe07070db70addd4dLL, 0x7c3e3ef83eebd597LL, 0x71b5b5e2b5df6aaeLL, 0xcc666683661d7bd1LL,
	0x9048483b48763ee6LL, 0x0603030c03181b1eLL, 0xf7f6f6f5f6f10706LL, 0x1c0e0e380e707e6cLL,
	0xc261619f612544e7LL, 0x6a3535d435b386d9LL, 0xae575747578ed920LL, 0x69b9b9d2b9bf06d6LL,
	0x1786862e865cda4bLL, 0x99c1c129c15293cbLL, 0x3a1d1d741de8f5d2LL, 0x279e9e4e9e9c02bbLL,
	0xd9e1e1a9e149a890LL, 0xebf8f8cdf881796aLL, 0x2b98985698ac3487LL, 0x22111144118899aaLL,
	0xd26969bf69650cb7LL, 0xa9d9d949d9924b3bLL, 0x078e8e0e8e1c921bLL, 0x3394946694cc58ffLL,
	0x2d9b9b5a9bb42f99LL, 0x3c1e1e781ef0eeccLL, 0x1587872a8754d341LL, 0xc9e9e989e909e0c0LL,
	0x87cece15ce2ae4adLL, 0xaa55554f559ecb34LL, 0x502828a0285b730bLL, 0xa5dfdf51dfa27d07LL,
	0x038c8c068c0c800fLL, 0x59a1a1b2a17fde26LL, 0x098989128924ad2dLL, 0x1a0d0d340d686572LL,
	0x65bfbfcabf8f30eaLL, 0xd7e6e6b5e67197a6LL, 0x84424213422664a2LL, 0xd06868bb686d05bdLL,
	0x8241411f413e7fbcLL, 0x2999995299a43d8dLL, 0x5a2d2db42d735e29LL, 0x1e0f0f3c0f787766LL,
	0x7bb0b0f6b0f7478cLL, 0xa854544b5496c23eLL, 0x6dbbbbdabbaf14c2LL, 0x2c16165816b0a69cLL
};

static const u64 TBLC3[256] = {
	0xf3c6636397633556LL, 0x35f87c7ceb7ccdb1LL, 0x7bee7777c77795e2LL, 0x03f67b7bf77bf58eLL,
	0x2efff2f2e5f2d123LL, 0xa3d66b6bb76b751eLL, 0x8bde6f6fa76f553aLL, 0xe391c5c539c572b7LL,
	0xfb603030c0309babLL, 0x0a02010104010809LL, 0xdbce676787671572LL, 0x15562b2bac2b4368LL,
	0x56e7fefed5feb14fLL, 0x57b5d7d771d7e235LL, 0x624dabab9aab2f84LL, 0x71ec7676c3769debLL,
	0x858fcaca05ca0ac0LL, 0x631f82823e827cfeLL, 0x9b89c9c909c912dbLL, 0x3ffa7d7def7dc5b8LL,
	0x7eeffafac5fa916bLL, 0x4cb259597f59fea7LL, 0x808e474707470e49LL, 0x3afbf0f0edf0c131LL,
	0x5e41adad82ad1fb2LL, 0x49b3d4d47dd4fa2eLL, 0x385fa2a2bea267c5LL, 0x4a45afaf8aaf0fa0LL,
	0xaf239c9c469c8c10LL, 0x0453a4a4a6a457f3LL, 0x59e47272d372bdcfLL, 0xc19bc0c02dc05a9aLL,
	0xba75b7b7eab7cf78LL, 0x48e1fdfdd9fda954LL, 0xc93d93937a93f467LL, 0x674c262698262b0dLL,
	0xc76c3636d836ab9dLL, 0x9d7e3f3ffc3fe3dcLL, 0x0cf5f7f7f1f7f90eLL, 0xb983cccc1dcc3af6LL,
	0xd3683434d034bb8fLL, 0x0e51a5a5a2a55ffaLL, 0xb8d1e5e5b9e5698cLL, 0x30f9f1f1e9f1c938LL,
	0x47e27171df71a5d4LL, 0x31abd8d84dd89a42LL, 0xf1623131c43193a2LL, 0x822a15155415a8bdLL,
	0x2808040410042024LL, 0xf795c7c731c762a5LL, 0x454623238c230320LL, 0xdf9dc3c321c34281LL,
	0xf03018186018c0d8LL, 0xeb3796966e96dc4aLL, 0x220a05051405282dLL, 0x932f9a9a5e9abc26LL,
	0x360e07071c07383fLL, 0xb424121248129082LL, 0x771b808036806cecLL, 0x8edfe2e2a5e251b3LL,
	0xd4cdebeb81eb19f2LL, 0x6d4e27279c272304LL, 0x987fb2b2feb2e755LL, 0x6fea7575cf7585f0LL,
	0x5a12090924094841LL, 0x691d83833a8374f7LL, 0x23582c2cb02c7b57LL, 0xe4341a1a681ad0caLL,
	0xee361b1b6c1bd8c3LL, 0x81dc6e6ea36e5d33LL, 0x52b45a5a735ae6bcLL, 0x2c5ba0a0b6a077d7LL,
	0x02a452525352a6f4LL, 0xb5763b3bec3bc3f8LL, 0x5db7d6d675d6ea3cLL, 0x927db3b3fab3ef5cLL,
	0x01522929a429537aLL, 0x84dde3e3a1e359baLL, 0x3d5e2f2fbc2f634cLL, 0x5f13848426844cc8LL,
	0x08a653535753aefdLL, 0x6bb9d1d169d1d203LL, 0x0000000000000000LL, 0xe8c1eded99ed29c4LL,
	0x5b40202080201b3bLL, 0x42e3fcfcddfca15dLL, 0x8679b1b1f2b1ff4eLL, 0x58b65b5b775beeb5LL,
	0xa9d46a6ab36a7d17LL, 0x8f8dcbcb01cb02c9LL, 0xe067bebecebe8739LL, 0xa1723939e439d3eaLL,
	0xf2944a4a334a662cLL, 0xce984c4c2b4c561aLL, 0x46b058587b58f6aeLL, 0xa785cfcf11cf22edLL,
	0x61bbd0d06dd0da0aLL, 0xfcc5efef91ef39d6LL, 0x684faaaa9eaa278dLL, 0x74edfbfbc1fb9962LL,
	0xa886434317432e6dLL, 0xc49a4d4d2f4d5e13LL, 0xe5663333cc3383b0LL, 0x55118585228544c1LL,
	0x948a45450f451e5bLL, 0x60e9f9f9c9f98970LL, 0x1404020208021012LL, 0x2bfe7f7fe77fd5aaLL,
	0x16a050505b50b6e6LL, 0x83783c3cf03cfbc7LL, 0xb1259f9f4a9f940bLL, 0x7c4ba8a896a8379fLL,
	0x1ca251515f51beefLL, 0x325da3a3baa36fccLL, 0xb68040401b403676LL, 0x11058f8f0a8f149bLL,
	0xc33f92927e92fc6eLL, 0xa5219d9d429d8419LL, 0xab703838e038dbe3LL, 0x18f1f5f5f9f5e91cLL,
	0xf463bcbcc6bc972bLL, 0xb077b6b6eeb6c771LL, 0x25afdada45da8a50LL, 0x5142212184211332LL,
	0xa020101040108090LL, 0x5ce5ffffd1ffb946LL, 0x24fdf3f3e1f3d92aLL, 0x75bfd2d265d2ca18LL,
	0xb381cdcd19cd32ffLL, 0x78180c0c300c606cLL, 0xbe2613134c13988bLL, 0xe2c3ecec9dec21cdLL,
	0x70be5f5f675fce91LL, 0xe13597976a97d443LL, 0x9e8844440b441652LL, 0x962e17175c17b8afLL,
	0xe993c4c43dc47abeLL, 0x1a55a7a7aaa74fe8LL, 0x21fc7e7ee37edda3LL, 0x897a3d3df43df3ceLL,
	0xc5c864648b640d69LL, 0x64ba5d5d6f5dde83LL, 0xfa3219196419c8d1LL, 0x53e67373d773b5c6LL,
	0xedc060609b602d4dLL, 0x7d198181328164e5LL, 0xd09e4f4f274f4e01LL, 0x19a3dcdc5ddcba66LL,
	0x4f44222288220b29LL, 0x1f542a2aa82a4b61LL, 0xd73b90907690ec7cLL, 0x270b888816882ca4LL,
	0x8a8c464603460640LL, 0xf6c7eeee95ee31dfLL, 0xdc6bb8b8d6b8b70fLL, 0x882814145014a0b4LL,
	0x0da7dede55deaa74LL, 0x7abc5e5e635ec698LL, 0x4e160b0b2c0b5853LL, 0x2faddbdb41db8259LL,
	0x9adbe0e0ade041a1LL, 0xef643232c8328bb9LL, 0xbf743a3ae83acbf1LL, 0x44140a0a280a505aLL,
	0xec9249493f497e37LL, 0x3c0c060618063036LL, 0x7348242490243b1fLL, 0x6eb85c5c6b5cd68aLL,
	0xd59fc2c225c24a88LL, 0x7fbdd3d361d3c211LL, 0x5443acac86ac17bbLL, 0xf9c4626293623d5fLL,
	0xdd3991917291e475LL, 0xf53195956295c451LL, 0xb2d3e4e4bde46185LL, 0x17f27979ff79e59cLL,
	0xacd5e7e7b1e7799eLL, 0x918bc8c80dc81ad2LL, 0xcd6e3737dc37a394LL, 0x9fda6d6daf6d4528LL,
	0x05018d8d028d0489LL, 0x43b1d5d579d5f227LL, 0xda9c4e4e234e4608LL, 0x7649a9a992a93f96LL,
	0x95d86c6cab6c4d21LL, 0x2aac5656435686d0LL, 0x12f3f4f4fdf4e115LL, 0xdecfeaea85ea11fbLL,
	0xcfca65658f650560LL, 0x09f47a7af37afd87LL, 0x4047aeae8eae07a9LL, 0x5010080820084048LL,
	0xc86fbabadebaa71dLL, 0x1df07878fb78ed95LL, 0x794a252594253316LL, 0x375c2e2eb82e6b45LL,
	0xd8381c1c701ce0fcLL, 0x1057a6a6aea647e1LL, 0xa473b4b4e6b4d763LL, 0xfd97c6c635c66aacLL,
	0xcacbe8e88de801e9LL, 0x13a1dddd59ddb26fLL, 0x65e87474cb748df9LL, 0xc63e1f1f7c1ff8e7LL,
	0xf8964b4b374b6e25LL, 0xfe61bdbdc2bd9f22LL, 0x390d8b8b1a8b34bfLL, 0x330f8a8a1e8a3cb6LL,
	0x4de07070db70adddLL, 0x977c3e3ef83eebd5LL, 0xae71b5b5e2b5df6aLL, 0xd1cc666683661d7bLL,
	0xe69048483b48763eLL, 0x1e0603030c03181bLL, 0x06f7f6f6f5f6f107LL, 0x6c1c0e0e380e707eLL,
	0xe7c261619f612544LL, 0xd96a3535d435b386LL, 0x20ae575747578ed9LL, 0xd669b9b9d2b9bf06LL,
	0x4b1786862e865cdaLL, 0xcb99c1c129c15293LL, 0xd23a1d1d741de8f5LL, 0xbb279e9e4e9e9c02LL,
	0x90d9e1e1a9e149a8LL, 0x6aebf8f8cdf88179LL, 0x872b98985698ac34LL, 0xaa22111144118899LL,
	0xb7d26969bf69650cLL, 0x3ba9d9d949d9924bLL, 0x1b078e8e0e8e1c92LL, 0xff3394946694cc58LL,
	0x992d9b9b5a9bb42fLL, 0xcc3c1e1e781ef0eeLL, 0x411587872a8754d3LL, 0xc0c9e9e989e909e0LL,
	0xad87cece15ce2ae4LL, 0x34aa55554f559ecbLL, 0x0b502828a0285b73LL, 0x07a5dfdf51dfa27dLL,
	0x0f038c8c068c0c80LL, 0x2659a1a1b2a17fdeLL, 0x2d098989128924adLL, 0x721a0d0d340d6865LL,
	0xea65bfbfcabf8f30LL, 0xa6d7e6e6b5e67197LL, 0xa284424213422664LL, 0xbdd06868bb686d05LL,
	0xbc8241411f413e7fLL, 0x8d2999995299a43dLL, 0x295a2d2db42d735eLL, 0x661e0f0f3c0f7877LL,
	0x8c7bb0b0f6b0f747LL, 0x3ea854544b5496c2LL, 0xc26dbbbbdabbaf14LL, 0x9c2c16165816b0a6LL
};

static const u64 TBLC4[256] = {
	0x56f3c66363976335LL, 0xb135f87c7ceb7ccdLL, 0xe27bee7777c77795LL, 0x8e03f67b7bf77bf5LL,
	0x232efff2f2e5f2d1LL, 0x1ea3d66b6bb76b75LL, 0x3a8bde6f6fa76f55LL, 0xb7e391c5c539c572LL,
	0xabfb603030c0309bLL, 0x090a020101040108LL, 0x72dbce6767876715LL, 0x6815562b2bac2b43LL,
	0x4f56e7fefed5feb1LL, 0x3557b5d7d771d7e2LL, 0x84624dabab9aab2fLL, 0xeb71ec7676c3769dLL,
	0xc0858fcaca05ca0aLL, 0xfe631f82823e827cLL, 0xdb9b89c9c909c912LL, 0xb83ffa7d7def7dc5LL,
	0x6b7eeffafac5fa91LL, 0xa74cb259597f59feLL, 0x49808e474707470eLL, 0x313afbf0f0edf0c1LL,
	0xb25e41adad82ad1fLL, 0x2e49b3d4d47dd4faLL, 0xc5385fa2a2bea267LL, 0xa04a45afaf8aaf0fLL,
	0x10af239c9c469c8cLL, 0xf30453a4a4a6a457LL, 0xcf59e47272d372bdLL, 0x9ac19bc0c02dc05aLL,
	0x78ba75b7b7eab7cfLL, 0x5448e1fdfdd9fda9LL, 0x67c93d93937a93f4LL, 0x0d674c262698262bLL,
	0x9dc76c3636d836abLL, 0xdc9d7e3f3ffc3fe3LL, 0x0e0cf5f7f7f1f7f9LL, 0xf6b983cccc1dcc3aLL,
	0x8fd3683434d034bbLL, 0xfa0e51a5a5a2a55fLL, 0x8cb8d1e5e5b9e569LL, 0x3830f9f1f1e9f1c9LL,
	0xd447e27171df71a5LL, 0x4231abd8d84dd89aLL, 0xa2f1623131c43193LL, 0xbd822a15155415a8LL,
	0x2428080404100420LL, 0xa5f795c7c731c762LL, 0x20454623238c2303LL, 0x81df9dc3c321c342LL,
	0xd8f03018186018c0LL, 0x4aeb3796966e96dcLL, 0x2d220a0505140528LL, 0x26932f9a9a5e9abcLL,
	0x3f360e07071c0738LL, 0x82b4241212481290LL, 0xec771b808036806cLL, 0xb38edfe2e2a5e251LL,
	0xf2d4cdebeb81eb19LL, 0x046d4e27279c2723LL, 0x55987fb2b2feb2e7LL, 0xf06fea7575cf7585LL,
	0x415a120909240948LL, 0xf7691d83833a8374LL, 0x5723582c2cb02c7bLL, 0xcae4341a1a681ad0LL,
	0xc3ee361b1b6c1bd8LL, 0x3381dc6e6ea36e5dLL, 0xbc52b45a5a735ae6LL, 0xd72c5ba0a0b6a077LL,
	0xf402a452525352a6LL, 0xf8b5763b3bec3bc3LL, 0x3c5db7d6d675d6eaLL, 0x5c927db3b3fab3efLL,
	0x7a01522929a42953LL, 0xba84dde3e3a1e359LL, 0x4c3d5e2f2fbc2f63LL, 0xc85f13848426844cLL,
	0xfd08a653535753aeLL, 0x036bb9d1d169d1d2LL, 0x0000000000000000LL, 0xc4e8c1eded99ed29LL,
	0x3b5b40202080201bLL, 0x5d42e3fcfcddfca1LL, 0x4e8679b1b1f2b1ffLL, 0xb558b65b5b775beeLL,
	0x17a9d46a6ab36a7dLL, 0xc98f8dcbcb01cb02LL, 0x39e067bebecebe87LL, 0xeaa1723939e439d3LL,
	0x2cf2944a4a334a66LL, 0x1ace984c4c2b4c56LL, 0xae46b058587b58f6LL, 0xeda785cfcf11cf22LL,
	0x0a61bbd0d06dd0daLL, 0xd6fcc5efef91ef39LL, 0x8d684faaaa9eaa27LL, 0x6274edfbfbc1fb99LL,
	0x6da886434317432eLL, 0x13c49a4d4d2f4d5eLL, 0xb0e5663333cc3383LL, 0xc155118585228544LL,
	0x5b948a45450f451eLL, 0x7060e9f9f9c9f989LL, 0x1214040202080210LL, 0xaa2bfe7f7fe77fd5LL,
	0xe616a050505b50b6LL, 0xc783783c3cf03cfbLL, 0x0bb1259f9f4a9f94LL, 0x9f7c4ba8a896a837LL,
	0xef1ca251515f51beLL, 0xcc325da3a3baa36fLL, 0x76b68040401b4036LL, 0x9b11058f8f0a8f14LL,
	0x6ec33f92927e92fcLL, 0x19a5219d9d429d84LL, 0xe3ab703838e038dbLL, 0x1c18f1f5f5f9f5e9LL,
	0x2bf463bcbcc6bc97LL, 0x71b077b6b6eeb6c7LL, 0x5025afdada45da8aLL, 0x3251422121842113LL,
	0x90a0201010401080LL, 0x465ce5ffffd1ffb9LL, 0x2a24fdf3f3e1f3d9LL, 0x1875bfd2d265d2caLL,
	0xffb381cdcd19cd32LL, 0x6c78180c0c300c60LL, 0x8bbe2613134c1398LL, 0xcde2c3ecec9dec21LL,
	0x9170be5f5f675fceLL, 0x43e13597976a97d4LL, 0x529e8844440b4416LL, 0xaf962e17175c17b8LL,
	0xbee993c4c43dc47aLL, 0xe81a55a7a7aaa74fLL, 0xa321fc7e7ee37eddLL, 0xce897a3d3df43df3LL,
	0x69c5c864648b640dLL, 0x8364ba5d5d6f5ddeLL, 0xd1fa3219196419c8LL, 0xc653e67373d773b5LL,
	0x4dedc060609b602dLL, 0xe57d198181328164LL, 0x01d09e4f4f274f4eLL, 0x6619a3dcdc5ddcbaLL,
	0x294f44222288220bLL, 0x611f542a2aa82a4bLL, 0x7cd73b90907690ecLL, 0xa4270b888816882cLL,
	0x408a8c4646034606LL, 0xdff6c7eeee95ee31LL, 0x0fdc6bb8b8d6b8b7LL, 0xb4882814145014a0LL,
	0x740da7dede55deaaLL, 0x987abc5e5e635ec6LL, 0x534e160b0b2c0b58LL, 0x592faddbdb41db82LL,
	0xa19adbe0e0ade041LL, 0xb9ef643232c8328bLL, 0xf1bf743a3ae83acbLL, 0x5a44140a0a280a50LL,
	0x37ec9249493f497eLL, 0x363c0c0606180630LL, 0x1f7348242490243bLL, 0x8a6eb85c5c6b5cd6LL,
	0x88d59fc2c225c24aLL, 0x117fbdd3d361d3c2LL, 0xbb5443acac86ac17LL, 0x5ff9c4626293623dLL,
	0x75dd3991917291e4LL, 0x51f53195956295c4LL, 0x85b2d3e4e4bde461LL, 0x9c17f27979ff79e5LL,
	0x9eacd5e7e7b1e779LL, 0xd2918bc8c80dc81aLL, 0x94cd6e3737dc37a3LL, 0x289fda6d6daf6d45LL,
	0x8905018d8d028d04LL, 0x2743b1d5d579d5f2LL, 0x08da9c4e4e234e46LL, 0x967649a9a992a93fLL,
	0x2195d86c6cab6c4dLL, 0xd02aac5656435686LL, 0x1512f3f4f4fdf4e1LL, 0xfbdecfeaea85ea11LL,
	0x60cfca65658f6505LL, 0x8709f47a7af37afdLL, 0xa94047aeae8eae07LL, 0x4850100808200840LL,
	0x1dc86fbabadebaa7LL, 0x951df07878fb78edLL, 0x16794a2525942533LL, 0x45375c2e2eb82e6bLL,
	0xfcd8381c1c701ce0LL, 0xe11057a6a6aea647LL, 0x63a473b4b4e6b4d7LL, 0xacfd97c6c635c66aLL,
	0xe9cacbe8e88de801LL, 0x6f13a1dddd59ddb2LL, 0xf965e87474cb748dLL, 0xe7c63e1f1f7c1ff8LL,
	0x25f8964b4b374b6eLL, 0x22fe61bdbdc2bd9fLL, 0xbf390d8b8b1a8b34LL, 0xb6330f8a8a1e8a3cLL,
	0xdd4de07070db70adLL, 0xd5977c3e3ef83eebLL, 0x6aae71b5b5e2b5dfLL, 0x7bd1cc666683661dLL,
	0x3ee69048483b4876LL, 0x1b1e0603030c0318LL, 0x0706f7f6f6f5f6f1LL, 0x7e6c1c0e0e380e70LL,
	0x44e7c261619f6125LL, 0x86d96a3535d435b3LL, 0xd920ae575747578eLL, 0x06d669b9b9d2b9bfLL,
	0xda4b1786862e865cLL, 0x93cb99c1c129c152LL, 0xf5d23a1d1d741de8LL, 0x02bb279e9e4e9e9cLL,
	0xa890d9e1e1a9e149LL, 0x796aebf8f8cdf881LL, 0x34872b98985698acLL, 0x99aa221111441188LL,
	0x0cb7d26969bf6965LL, 0x4b3ba9d9d949d992LL, 0x921b078e8e0e8e1cLL, 0x58ff3394946694ccLL,
	0x2f992d9b9b5a9bb4LL, 0xeecc3c1e1e781ef0LL, 0xd3411587872a8754LL, 0xe0c0c9e9e989e909LL,
	0xe4ad87cece15ce2aLL, 0xcb34aa55554f559eLL, 0x730b502828a0285bLL, 0x7d07a5dfdf51dfa2LL,
	0x800f038c8c068c0cLL, 0xde2659a1a1b2a17fLL, 0xad2d098989128924LL, 0x65721a0d0d340d68LL,
	0x30ea65bfbfcabf8fLL, 0x97a6d7e6e6b5e671LL, 0x64a2844242134226LL, 0x05bdd06868bb686dLL,
	0x7fbc8241411f413eLL, 0x3d8d2999995299a4LL, 0x5e295a2d2db42d73LL, 0x77661e0f0f3c0f78LL,
	0x478c7bb0b0f6b0f7LL, 0xc23ea854544b5496LL, 0x14c26dbbbbdabbafLL, 0xa69c2c16165816b0LL
};

static const u64 TBLC5[256] = {
	0x3556f3c663639763LL, 0xcdb135f87c7ceb7cLL, 0x95e27bee7777c777LL, 0xf58e03f67b7bf77bLL,
	0xd1232efff2f2e5f2LL, 0x751ea3d66b6bb76bLL, 0x553a8bde6f6fa76fLL, 0x72b7e391c5c539c5LL,
	0x9babfb603030c030LL, 0x08090a0201010401LL, 0x1572dbce67678767LL, 0x436815562b2bac2bLL,
	0xb14f56e7fefed5feLL, 0xe23557b5d7d771d7LL, 0x2f84624dabab9aabLL, 0x9deb71ec7676c376LL,
	0x0ac0858fcaca05caLL, 0x7cfe631f82823e82LL, 0x12db9b89c9c909c9LL, 0xc5b83ffa7d7def7dLL,
	0x916b7eeffafac5faLL, 0xfea74cb259597f59LL, 0x0e49808e47470747LL, 0xc1313afbf0f0edf0LL,
	0x1fb25e41adad82adLL, 0xfa2e49b3d4d47dd4LL, 0x67c5385fa2a2bea2LL, 0x0fa04a45afaf8aafLL,
	0x8c10af239c9c469cLL, 0x57f30453a4a4a6a4LL, 0xbdcf59e47272d372LL, 0x5a9ac19bc0c02dc0LL,
	0xcf78ba75b7b7eab7LL, 0xa95448e1fdfdd9fdLL, 0xf467c93d93937a93LL, 0x2b0d674c26269826LL,
	0xab9dc76c3636d836LL, 0xe3dc9d7e3f3ffc3fLL, 0xf90e0cf5f7f7f1f7LL, 0x3af6b983cccc1dccLL,
	0xbb8fd3683434d034LL, 0x5ffa0e51a5a5a2a5LL, 0x698cb8d1e5e5b9e5LL, 0xc93830f9f1f1e9f1LL,
	0xa5d447e27171df71LL, 0x9a4231abd8d84dd8LL, 0x93a2f1623131c431LL, 0xa8bd822a15155415LL,
	0x2024280804041004LL, 0x62a5f795c7c731c7LL, 0x0320454623238c23LL, 0x4281df9dc3c321c3LL,
	0xc0d8f03018186018LL, 0xdc4aeb3796966e96LL, 0x282d220a05051405LL, 0xbc26932f9a9a5e9aLL,
	0x383f360e07071c07LL, 0x9082b42412124812LL, 0x6cec771b80803680LL, 0x51b38edfe2e2a5e2LL,
	0x19f2d4cdebeb81ebLL, 0x23046d4e27279c27LL, 0xe755987fb2b2feb2LL, 0x85f06fea7575cf75LL,
	0x48415a1209092409LL, 0x74f7691d83833a83LL, 0x7b5723582c2cb02cLL, 0xd0cae4341a1a681aLL,
	0xd8c3ee361b1b6c1bLL, 0x5d3381dc6e6ea36eLL, 0xe6bc52b45a5a735aLL, 0x77d72c5ba0a0b6a0LL,
	0xa6f402a452525352LL, 0xc3f8b5763b3bec3bLL, 0xea3c5db7d6d675d6LL, 0xef5c927db3b3fab3LL,
	0x537a01522929a429LL, 0x59ba84dde3e3a1e3LL, 0x634c3d5e2f2fbc2fLL, 0x4cc85f1384842684LL,
	0xaefd08a653535753LL, 0xd2036bb9d1d169d1LL, 0x0000000000000000LL, 0x29c4e8c1eded99edLL,
	0x1b3b5b4020208020LL, 0xa15d42e3fcfcddfcLL, 0xff4e8679b1b1f2b1LL, 0xeeb558b65b5b775bLL,
	0x7d17a9d46a6ab36aLL, 0x02c98f8dcbcb01cbLL, 0x8739e067bebecebeLL, 0xd3eaa1723939e439LL,
	0x662cf2944a4a334aLL, 0x561ace984c4c2b4cLL, 0xf6ae46b058587b58LL, 0x22eda785cfcf11cfLL,
	0xda0a61bbd0d06dd0LL, 0x39d6fcc5efef91efLL, 0x278d684faaaa9eaaLL, 0x996274edfbfbc1fbLL,
	0x2e6da88643431743LL, 0x5e13c49a4d4d2f4dLL, 0x83b0e5663333cc33LL, 0x44c1551185852285LL,
	0x1e5b948a45450f45LL, 0x897060e9f9f9c9f9LL, 0x1012140402020802LL, 0xd5aa2bfe7f7fe77fLL,
	0xb6e616a050505b50LL, 0xfbc783783c3cf03cLL, 0x940bb1259f9f4a9fLL, 0x379f7c4ba8a896a8LL,
	0xbeef1ca251515f51LL, 0x6fcc325da3a3baa3LL, 0x3676b68040401b40LL, 0x149b11058f8f0a8fLL,
	0xfc6ec33f92927e92LL, 0x8419a5219d9d429dLL, 0xdbe3ab703838e038LL, 0xe91c18f1f5f5f9f5LL,
	0x972bf463bcbcc6bcLL, 0xc771b077b6b6eeb6LL, 0x8a5025afdada45daLL, 0x1332514221218421LL,
	0x8090a02010104010LL, 0xb9465ce5ffffd1ffLL, 0xd92a24fdf3f3e1f3LL, 0xca1875bfd2d265d2LL,
	0x32ffb381cdcd19cdLL, 0x606c78180c0c300cLL, 0x988bbe2613134c13LL, 0x21cde2c3ecec9decLL,
	0xce9170be5f5f675fLL, 0xd443e13597976a97LL, 0x16529e8844440b44LL, 0xb8af962e17175c17LL,
	0x7abee993c4c43dc4LL, 0x4fe81a55a7a7aaa7LL, 0xdda321fc7e7ee37eLL, 0xf3ce897a3d3df43dLL,
	0x0d69c5c864648b64LL, 0xde8364ba5d5d6f5dLL, 0xc8d1fa3219196419LL, 0xb5c653e67373d773LL,
	0x2d4dedc060609b60LL, 0x64e57d1981813281LL, 0x4e01d09e4f4f274fLL, 0xba6619a3dcdc5ddcLL,
	0x0b294f4422228822LL, 0x4b611f542a2aa82aLL, 0xec7cd73b90907690LL, 0x2ca4270b88881688LL,
	0x06408a8c46460346LL, 0x31dff6c7eeee95eeLL, 0xb70fdc6bb8b8d6b8LL, 0xa0b4882814145014LL,
	0xaa740da7dede55deLL, 0xc6987abc5e5e635eLL, 0x58534e160b0b2c0bLL, 0x82592faddbdb41dbLL,
	0x41a19adbe0e0ade0LL, 0x8bb9ef643232c832LL, 0xcbf1bf743a3ae83aLL, 0x505a44140a0a280aLL,
	0x7e37ec9249493f49LL, 0x30363c0c06061806LL, 0x3b1f734824249024LL, 0xd68a6eb85c5c6b5cLL,
	0x4a88d59fc2c225c2LL, 0xc2117fbdd3d361d3LL, 0x17bb5443acac86acLL, 0x3d5ff9c462629362LL,
	0xe475dd3991917291LL, 0xc451f53195956295LL, 0x6185b2d3e4e4bde4LL, 0xe59c17f27979ff79LL,
	0x799eacd5e7e7b1e7LL, 0x1ad2918bc8c80dc8LL, 0xa394cd6e3737dc37LL, 0x45289fda6d6daf6dLL,
	0x048905018d8d028dLL, 0xf22743b1d5d579d5LL, 0x4608da9c4e4e234eLL, 0x3f967649a9a992a9LL,
	0x4d2195d86c6cab6cLL, 0x86d02aac56564356LL, 0xe11512f3f4f4fdf4LL, 0x11fbdecfeaea85eaLL,
	0x0560cfca65658f65LL, 0xfd8709f47a7af37aLL, 0x07a94047aeae8eaeLL, 0x4048501008082008LL,
	0xa71dc86fbabadebaLL, 0xed951df07878fb78LL, 0x3316794a25259425LL, 0x6b45375c2e2eb82eLL,
	0xe0fcd8381c1c701cLL, 0x47e11057a6a6aea6LL, 0xd763a473b4b4e6b4LL, 0x6aacfd97c6c635c6LL,
	0x01e9cacbe8e88de8LL, 0xb26f13a1dddd59ddLL, 0x8df965e87474cb74LL, 0xf8e7c63e1f1f7c1fLL,
	0x6e25f8964b4b374bLL, 0x9f22fe61bdbdc2bdLL, 0x34bf390d8b8b1a8bLL, 0x3cb6330f8a8a1e8aLL,
	0xaddd4de07070db70LL, 0xebd5977c3e3ef83eLL, 0xdf6aae71b5b5e2b5LL, 0x1d7bd1cc66668366LL,
	0x763ee69048483b48LL, 0x181b1e0603030c03LL, 0xf10706f7f6f6f5f6LL, 0x707e6c1c0e0e380eLL,
	0x2544e7c261619f61LL, 0xb386d96a3535d435LL, 0x8ed920ae57574757LL, 0xbf06d669b9b9d2b9LL,
	0x5cda4b1786862e86LL, 0x5293cb99c1c129c1LL, 0xe8f5d23a1d1d741dLL, 0x9c02bb279e9e4e9eLL,
	0x49a890d9e1e1a9e1LL, 0x81796aebf8f8cdf8LL, 0xac34872b98985698LL, 0x8899aa2211114411LL,
	0x650cb7d26969bf69LL, 0x924b3ba9d9d949d9LL, 0x1c921b078e8e0e8eLL, 0xcc58ff3394946694LL,
	0xb42f992d9b9b5a9bLL, 0xf0eecc3c1e1e781eLL, 0x54d3411587872a87LL, 0x09e0c0c9e9e989e9LL,
	0x2ae4ad87cece15ceLL, 0x9ecb34aa55554f55LL, 0x5b730b502828a028LL, 0xa27d07a5dfdf51dfLL,
	0x0c800f038c8c068cLL, 0x7fde2659a1a1b2a1LL, 0x24ad2d0989891289LL, 0x6865721a0d0d340dLL,
	0x8f30ea65bfbfcabfLL, 0x7197a6d7e6e6b5e6LL, 0x2664a28442421342LL, 0x6d05bdd06868bb68LL,
	0x3e7fbc8241411f41LL, 0xa43d8d2999995299LL, 0x735e295a2d2db42dLL, 0x7877661e0f0f3c0fLL,
	0xf7478c7bb0b0f6b0LL, 0x96c23ea854544b54LL, 0xaf14c26dbbbbdabbLL, 0xb0a69c2c16165816LL
};

static const u64 TBLC6[256] = {
	0x633556f3c6636397LL, 0x7ccdb135f87c7cebLL, 0x7795e27bee7777c7LL, 0x7bf58e03f67b7bf7LL,
	0xf2d1232efff2f2e5LL, 0x6b751ea3d66b6bb7LL, 0x6f553a8bde6f6fa7LL, 0xc572b7e391c5c539LL,
	0x309babfb603030c0LL, 0x0108090a02010104LL, 0x671572dbce676787LL, 0x2b436815562b2bacLL,
	0xfeb14f56e7fefed5LL, 0xd7e23557b5d7d771LL, 0xab2f84624dabab9aLL, 0x769deb71ec7676c3LL,
	0xca0ac0858fcaca05LL, 0x827cfe631f82823eLL, 0xc912db9b89c9c909LL, 0x7dc5b83ffa7d7defLL,
	0xfa916b7eeffafac5LL, 0x59fea74cb259597fLL, 0x470e49808e474707LL, 0xf0c1313afbf0f0edLL,
	0xad1fb25e41adad82LL, 0xd4fa2e49b3d4d47dLL, 0xa267c5385fa2a2beLL, 0xaf0fa04a45afaf8aLL,
	0x9c8c10af239c9c46LL, 0xa457f30453a4a4a6LL, 0x72bdcf59e47272d3LL, 0xc05a9ac19bc0c02dLL,
	0xb7cf78ba75b7b7eaLL, 0xfda95448e1fdfdd9LL, 0x93f467c93d93937aLL, 0x262b0d674c262698LL,
	0x36ab9dc76c3636d8LL, 0x3fe3dc9d7e3f3ffcLL, 0xf7f90e0cf5f7f7f1LL, 0xcc3af6b983cccc1dLL,
	0x34bb8fd3683434d0LL, 0xa55ffa0e51a5a5a2LL, 0xe5698cb8d1e5e5b9LL, 0xf1c93830f9f1f1e9LL,
	0x71a5d447e27171dfLL, 0xd89a4231abd8d84dLL, 0x3193a2f1623131c4LL, 0x15a8bd822a151554LL,
	0x0420242808040410LL, 0xc762a5f795c7c731LL, 0x230320454623238cLL, 0xc34281df9dc3c321LL,
	0x18c0d8f030181860LL, 0x96dc4aeb3796966eLL, 0x05282d220a050514LL, 0x9abc26932f9a9a5eLL,
	0x07383f360e07071cLL, 0x129082b424121248LL, 0x806cec771b808036LL, 0xe251b38edfe2e2a5LL,
	0xeb19f2d4cdebeb81LL, 0x2723046d4e27279cLL, 0xb2e755987fb2b2feLL, 0x7585f06fea7575cfLL,
	0x0948415a12090924LL, 0x8374f7691d83833aLL, 0x2c7b5723582c2cb0LL, 0x1ad0cae4341a1a68LL,
	0x1bd8c3ee361b1b6cLL, 0x6e5d3381dc6e6ea3LL, 0x5ae6bc52b45a5a73LL, 0xa077d72c5ba0a0b6LL,
	0x52a6f402a4525253LL, 0x3bc3f8b5763b3becLL, 0xd6ea3c5db7d6d675LL, 0xb3ef5c927db3b3faLL,
	0x29537a01522929a4LL, 0xe359ba84dde3e3a1LL, 0x2f634c3d5e2f2fbcLL, 0x844cc85f13848426LL,
	0x53aefd08a6535357LL, 0xd1d2036bb9d1d169LL, 0x0000000000000000LL, 0xed29c4e8c1eded99LL,
	0x201b3b5b40202080LL, 0xfca15d42e3fcfcddLL, 0xb1ff4e8679b1b1f2LL, 0x5beeb558b65b5b77LL,
	0x6a7d17a9d46a6ab3LL, 0xcb02c98f8dcbcb01LL, 0xbe8739e067bebeceLL, 0x39d3eaa1723939e4LL,
	0x4a662cf2944a4a33LL, 0x4c561ace984c4c2bLL, 0x58f6ae46b058587bLL, 0xcf22eda785cfcf11LL,
	0xd0da0a61bbd0d06dLL, 0xef39d6fcc5efef91LL, 0xaa278d684faaaa9eLL, 0xfb996274edfbfbc1LL,
	0x432e6da886434317LL, 0x4d5e13c49a4d4d2fLL, 0x3383b0e5663333ccLL, 0x8544c15511858522LL,
	0x451e5b948a45450fLL, 0xf9897060e9f9f9c9LL, 0x0210121404020208LL, 0x7fd5aa2bfe7f7fe7LL,
	0x50b6e616a050505bLL, 0x3cfbc783783c3cf0LL, 0x9f940bb1259f9f4aLL, 0xa8379f7c4ba8a896LL,
	0x51beef1ca251515fLL, 0xa36fcc325da3a3baLL, 0x403676b68040401bLL, 0x8f149b11058f8f0aLL,
	0x92fc6ec33f92927eLL, 0x9d8419a5219d9d42LL, 0x38dbe3ab703838e0LL, 0xf5e91c18f1f5f5f9LL,
	0xbc972bf463bcbcc6LL, 0xb6c771b077b6b6eeLL, 0xda8a5025afdada45LL, 0x2113325142212184LL,
	0x108090a020101040LL, 0xffb9465ce5ffffd1LL, 0xf3d92a24fdf3f3e1LL, 0xd2ca1875bfd2d265LL,
	0xcd32ffb381cdcd19LL, 0x0c606c78180c0c30LL, 0x13988bbe2613134cLL, 0xec21cde2c3ecec9dLL,
	0x5fce9170be5f5f67LL, 0x97d443e13597976aLL, 0x4416529e8844440bLL, 0x17b8af962e17175cLL,
	0xc47abee993c4c43dLL, 0xa74fe81a55a7a7aaLL, 0x7edda321fc7e7ee3LL, 0x3df3ce897a3d3df4LL,
	0x640d69c5c864648bLL, 0x5dde8364ba5d5d6fLL, 0x19c8d1fa32191964LL, 0x73b5c653e67373d7LL,
	0x602d4dedc060609bLL, 0x8164e57d19818132LL, 0x4f4e01d09e4f4f27LL, 0xdcba6619a3dcdc5dLL,
	0x220b294f44222288LL, 0x2a4b611f542a2aa8LL, 0x90ec7cd73b909076LL, 0x882ca4270b888816LL,
	0x4606408a8c464603LL, 0xee31dff6c7eeee95LL, 0xb8b70fdc6bb8b8d6LL, 0x14a0b48828141450LL,
	0xdeaa740da7dede55LL, 0x5ec6987abc5e5e63LL, 0x0b58534e160b0b2cLL, 0xdb82592faddbdb41LL,
	0xe041a19adbe0e0adLL, 0x328bb9ef643232c8LL, 0x3acbf1bf743a3ae8LL, 0x0a505a44140a0a28LL,
	0x497e37ec9249493fLL, 0x0630363c0c060618LL, 0x243b1f7348242490LL, 0x5cd68a6eb85c5c6bLL,
	0xc24a88d59fc2c225LL, 0xd3c2117fbdd3d361LL, 0xac17bb5443acac86LL, 0x623d5ff9c4626293LL,
	0x91e475dd39919172LL, 0x95c451f531959562LL, 0xe46185b2d3e4e4bdLL, 0x79e59c17f27979ffLL,
	0xe7799eacd5e7e7b1LL, 0xc81ad2918bc8c80dLL, 0x37a394cd6e3737dcLL, 0x6d45289fda6d6dafLL,
	0x8d048905018d8d02LL, 0xd5f22743b1d5d579LL, 0x4e4608da9c4e4e23LL, 0xa93f967649a9a992LL,
	0x6c4d2195d86c6cabLL, 0x5686d02aac565643LL, 0xf4e11512f3f4f4fdLL, 0xea11fbdecfeaea85LL,
	0x650560cfca65658fLL, 0x7afd8709f47a7af3LL, 0xae07a94047aeae8eLL, 0x0840485010080820LL,
	0xbaa71dc86fbabadeLL, 0x78ed951df07878fbLL, 0x253316794a252594LL, 0x2e6b45375c2e2eb8LL,
	0x1ce0fcd8381c1c70LL, 0xa647e11057a6a6aeLL, 0xb4d763a473b4b4e6LL, 0xc66aacfd97c6c635LL,
	0xe801e9cacbe8e88dLL, 0xddb26f13a1dddd59LL, 0x748df965e87474cbLL, 0x1ff8e7c63e1f1f7cLL,
	0x4b6e25f8964b4b37LL, 0xbd9f22fe61bdbdc2LL, 0x8b34bf390d8b8b1aLL, 0x8a3cb6330f8a8a1eLL,
	0x70addd4de07070dbLL, 0x3eebd5977c3e3ef8LL, 0xb5df6aae71b5b5e2LL, 0x661d7bd1cc666683LL,
	0x48763ee69048483bLL, 0x03181b1e0603030cLL, 0xf6f10706f7f6f6f5LL, 0x0e707e6c1c0e0e38LL,
	0x612544e7c261619fLL, 0x35b386d96a3535d4LL, 0x578ed920ae575747LL, 0xb9bf06d669b9b9d2LL,
	0x865cda4b1786862eLL, 0xc15293cb99c1c129LL, 0x1de8f5d23a1d1d74LL, 0x9e9c02bb279e9e4eLL,
	0xe149a890d9e1e1a9LL, 0xf881796aebf8f8cdLL, 0x98ac34872b989856LL, 0x118899aa22111144LL,
	0x69650cb7d26969bfLL, 0xd9924b3ba9d9d949LL, 0x8e1c921b078e8e0eLL, 0x94cc58ff33949466LL,
	0x9bb42f992d9b9b5aLL, 0x1ef0eecc3c1e1e78LL, 0x8754d3411587872aLL, 0xe909e0c0c9e9e989LL,
	0xce2ae4ad87cece15LL, 0x559ecb34aa55554fLL, 0x285b730b502828a0LL, 0xdfa27d07a5dfdf51LL,
	0x8c0c800f038c8c06LL, 0xa17fde2659a1a1b2LL, 0x8924ad2d09898912LL, 0x0d6865721a0d0d34LL,
	0xbf8f30ea65bfbfcaLL, 0xe67197a6d7e6e6b5LL, 0x422664a284424213LL, 0x686d05bdd06868bbLL,
	0x413e7fbc8241411fLL, 0x99a43d8d29999952LL, 0x2d735e295a2d2db4LL, 0x0f7877661e0f0f3cLL,
	0xb0f7478c7bb0b0f6LL, 0x5496c23ea854544bLL, 0xbbaf14c26dbbbbdaLL, 0x16b0a69c2c161658LL
};

static const u64 TBLC7[256] = {
	0x97633556f3c66363LL, 0xeb7ccdb135f87c7cLL, 0xc77795e27bee7777LL, 0xf77bf58e03f67b7bLL,
	0xe5f2d1232efff2f2LL, 0xb76b751ea3d66b6bLL, 0xa76f553a8bde6f6fLL, 0x39c572b7e391c5c5LL,
	0xc0309babfb603030LL, 0x040108090a020101LL, 0x87671572dbce6767LL, 0xac2b436815562b2bLL,
	0xd5feb14f56e7fefeLL, 0x71d7e23557b5d7d7LL, 0x9aab2f84624dababLL, 0xc3769deb71ec7676LL,
	0x05ca0ac0858fcacaLL, 0x3e827cfe631f8282LL, 0x09c912db9b89c9c9LL, 0xef7dc5b83ffa7d7dLL,
	0xc5fa916b7eeffafaLL, 0x7f59fea74cb25959LL, 0x07470e49808e4747LL, 0xedf0c1313afbf0f0LL,
	0x82ad1fb25e41adadLL, 0x7dd4fa2e49b3d4d4LL, 0xbea267c5385fa2a2LL, 0x8aaf0fa04a45afafLL,
	0x469c8c10af239c9cLL, 0xa6a457f30453a4a4LL, 0xd372bdcf59e47272LL, 0x2dc05a9ac19bc0c0LL,
	0xeab7cf78ba75b7b7LL, 0xd9fda95448e1fdfdLL, 0x7a93f467c93d9393LL, 0x98262b0d674c2626LL,
	0xd836ab9dc76c3636LL, 0xfc3fe3dc9d7e3f3fLL, 0xf1f7f90e0cf5f7f7LL, 0x1dcc3af6b983ccccLL,
	0xd034bb8fd3683434LL, 0xa2a55ffa0e51a5a5LL, 0xb9e5698cb8d1e5e5LL, 0xe9f1c93830f9f1f1LL,
	0xdf71a5d447e27171LL, 0x4dd89a4231abd8d8LL, 0xc43193a2f1623131LL, 0x5415a8bd822a1515LL,
	0x1004202428080404LL, 0x31c762a5f795c7c7LL, 0x8c23032045462323LL, 0x21c34281df9dc3c3LL,
	0x6018c0d8f0301818LL, 0x6e96dc4aeb379696LL, 0x1405282d220a0505LL, 0x5e9abc26932f9a9aLL,
	0x1c07383f360e0707LL, 0x48129082b4241212LL, 0x36806cec771b8080LL, 0xa5e251b38edfe2e2LL,
	0x81eb19f2d4cdebebLL, 0x9c2723046d4e2727LL, 0xfeb2e755987fb2b2LL, 0xcf7585f06fea7575LL,
	0x240948415a120909LL, 0x3a8374f7691d8383LL, 0xb02c7b5723582c2cLL, 0x681ad0cae4341a1aLL,
	0x6c1bd8c3ee361b1bLL, 0xa36e5d3381dc6e6eLL, 0x735ae6bc52b45a5aLL, 0xb6a077d72c5ba0a0LL,
	0x5352a6f402a45252LL, 0xec3bc3f8b5763b3bLL, 0x75d6ea3c5db7d6d6LL, 0xfab3ef5c927db3b3LL,
	0xa429537a01522929LL, 0xa1e359ba84dde3e3LL, 0xbc2f634c3d5e2f2fLL, 0x26844cc85f138484LL,
	0x5753aefd08a65353LL, 0x69d1d2036bb9d1d1LL, 0x0000000000000000LL, 0x99ed29c4e8c1ededLL,
	0x80201b3b5b402020LL, 0xddfca15d42e3fcfcLL, 0xf2b1ff4e8679b1b1LL, 0x775beeb558b65b5bLL,
	0xb36a7d17a9d46a6aLL, 0x01cb02c98f8dcbcbLL, 0xcebe8739e067bebeLL, 0xe439d3eaa1723939LL,
	0x334a662cf2944a4aLL, 0x2b4c561ace984c4cLL, 0x7b58f6ae46b05858LL, 0x11cf22eda785cfcfLL,
	0x6dd0da0a61bbd0d0LL, 0x91ef39d6fcc5efefLL, 0x9eaa278d684faaaaLL, 0xc1fb996274edfbfbLL,
	0x17432e6da8864343LL, 0x2f4d5e13c49a4d4dLL, 0xcc3383b0e5663333LL, 0x228544c155118585LL,
	0x0f451e5b948a4545LL, 0xc9f9897060e9f9f9LL, 0x0802101214040202LL, 0xe77fd5aa2bfe7f7fLL,
	0x5b50b6e616a05050LL, 0xf03cfbc783783c3cLL, 0x4a9f940bb1259f9fLL, 0x96a8379f7c4ba8a8LL,
	0x5f51beef1ca25151LL, 0xbaa36fcc325da3a3LL, 0x1b403676b6804040LL, 0x0a8f149b11058f8fLL,
	0x7e92fc6ec33f9292LL, 0x429d8419a5219d9dLL, 0xe038dbe3ab703838LL, 0xf9f5e91c18f1f5f5LL,
	0xc6bc972bf463bcbcLL, 0xeeb6c771b077b6b6LL, 0x45da8a5025afdadaLL, 0x8421133251422121LL,
	0x40108090a0201010LL, 0xd1ffb9465ce5ffffLL, 0xe1f3d92a24fdf3f3LL, 0x65d2ca1875bfd2d2LL,
	0x19cd32ffb381cdcdLL, 0x300c606c78180c0cLL, 0x4c13988bbe261313LL, 0x9dec21cde2c3ececLL,
	0x675fce9170be5f5fLL, 0x6a97d443e1359797LL, 0x0b4416529e884444LL, 0x5c17b8af962e1717LL,
	0x3dc47abee993c4c4LL, 0xaaa74fe81a55a7a7LL, 0xe37edda321fc7e7eLL, 0xf43df3ce897a3d3dLL,
	0x8b640d69c5c86464LL, 0x6f5dde8364ba5d5dLL, 0x6419c8d1fa321919LL, 0xd773b5c653e67373LL,
	0x9b602d4dedc06060LL, 0x328164e57d198181LL, 0x274f4e01d09e4f4fLL, 0x5ddcba6619a3dcdcLL,
	0x88220b294f442222LL, 0xa82a4b611f542a2aLL, 0x7690ec7cd73b9090LL, 0x16882ca4270b8888LL,
	0x034606408a8c4646LL, 0x95ee31dff6c7eeeeLL, 0xd6b8b70fdc6bb8b8LL, 0x5014a0b488281414LL,
	0x55deaa740da7dedeLL, 0x635ec6987abc5e5eLL, 0x2c0b58534e160b0bLL, 0x41db82592faddbdbLL,
	0xade041a19adbe0e0LL, 0xc8328bb9ef643232LL, 0xe83acbf1bf743a3aLL, 0x280a505a44140a0aLL,
	0x3f497e37ec924949LL, 0x180630363c0c0606LL, 0x90243b1f73482424LL, 0x6b5cd68a6eb85c5cLL,
	0x25c24a88d59fc2c2LL, 0x61d3c2117fbdd3d3LL, 0x86ac17bb5443acacLL, 0x93623d5ff9c46262LL,
	0x7291e475dd399191LL, 0x6295c451f5319595LL, 0xbde46185b2d3e4e4LL, 0xff79e59c17f27979LL,
	0xb1e7799eacd5e7e7LL, 0x0dc81ad2918bc8c8LL, 0xdc37a394cd6e3737LL, 0xaf6d45289fda6d6dLL,
	0x028d048905018d8dLL, 0x79d5f22743b1d5d5LL, 0x234e4608da9c4e4eLL, 0x92a93f967649a9a9LL,
	0xab6c4d2195d86c6cLL, 0x435686d02aac5656LL, 0xfdf4e11512f3f4f4LL, 0x85ea11fbdecfeaeaLL,
	0x8f650560cfca6565LL, 0xf37afd8709f47a7aLL, 0x8eae07a94047aeaeLL, 0x2008404850100808LL,
	0xdebaa71dc86fbabaLL, 0xfb78ed951df07878LL, 0x94253316794a2525LL, 0xb82e6b45375c2e2eLL,
	0x701ce0fcd8381c1cLL, 0xaea647e11057a6a6LL, 0xe6b4d763a473b4b4LL, 0x35c66aacfd97c6c6LL,
	0x8de801e9cacbe8e8LL, 0x59ddb26f13a1ddddLL, 0xcb748df965e87474LL, 0x7c1ff8e7c63e1f1fLL,
	0x374b6e25f8964b4bLL, 0xc2bd9f22fe61bdbdLL, 0x1a8b34bf390d8b8bLL, 0x1e8a3cb6330f8a8aLL,
	0xdb70addd4de07070LL, 0xf83eebd5977c3e3eLL, 0xe2b5df6aae71b5b5LL, 0x83661d7bd1cc6666LL,
	0x3b48763ee6904848LL, 0x0c03181b1e060303LL, 0xf5f6f10706f7f6f6LL, 0x380e707e6c1c0e0eLL,
	0x9f612544e7c26161LL, 0xd435b386d96a3535LL, 0x47578ed920ae5757LL, 0xd2b9bf06d669b9b9LL,
	0x2e865cda4b178686LL, 0x29c15293cb99c1c1LL, 0x741de8f5d23a1d1dLL, 0x4e9e9c02bb279e9eLL,
	0xa9e149a890d9e1e1LL, 0xcdf881796aebf8f8LL, 0x5698ac34872b9898LL, 0x44118899aa221111LL,
	0xbf69650cb7d26969LL, 0x49d9924b3ba9d9d9LL, 0x0e8e1c921b078e8eLL, 0x6694cc58ff339494LL,
	0x5a9bb42f992d9b9bLL, 0x781ef0eecc3c1e1eLL, 0x2a8754d341158787LL, 0x89e909e0c0c9e9e9LL,
	0x15ce2ae4ad87ceceLL, 0x4f559ecb34aa5555LL, 0xa0285b730b502828LL, 0x51dfa27d07a5dfdfLL,
	0x068c0c800f038c8cLL, 0xb2a17fde2659a1a1LL, 0x128924ad2d098989LL, 0x340d6865721a0d0dLL,
	0xcabf8f30ea65bfbfLL, 0xb5e67197a6d7e6e6LL, 0x13422664a2844242LL, 0xbb686d05bdd06868LL,
	0x1f413e7fbc824141LL, 0x5299a43d8d299999LL, 0xb42d735e295a2d2dLL, 0x3c0f7877661e0f0fLL,
	0xf6b0f7478c7bb0b0LL, 0x4b5496c23ea85454LL, 0xdabbaf14c26dbbbbLL, 0x5816b0a69c2c1616LL
};

static const u32 TBLD0[512]={
	0x00000000, 0xc66363a5, 0x00000000, 0xf87c7c84, 0x00000000, 0xee777799, 0x00000000, 0xf67b7b8d, 
	0x00000000, 0xfff2f20d, 0x00000000, 0xd66b6bbd, 0x00000000, 0xde6f6fb1, 0x00000000, 0x91c5c554, 
	0x00000000, 0x60303050, 0x00000000, 0x02010103, 0x00000000, 0xce6767a9, 0x00000000, 0x562b2b7d, 
	0x00000000, 0xe7fefe19, 0x00000000, 0xb5d7d762, 0x00000000, 0x4dababe6, 0x00000000, 0xec76769a, 
	0x00000000, 0x8fcaca45, 0x00000000, 0x1f82829d, 0x00000000, 0x89c9c940, 0x00000000, 0xfa7d7d87, 
	0x00000000, 0xeffafa15, 0x00000000, 0xb25959eb, 0x00000000, 0x8e4747c9, 0x00000000, 0xfbf0f00b, 
	0x00000000, 0x41adadec, 0x00000000, 0xb3d4d467, 0x00000000, 0x5fa2a2fd, 0x00000000, 0x45afafea, 
	0x00000000, 0x239c9cbf, 0x00000000, 0x53a4a4f7, 0x00000000, 0xe4727296, 0x00000000, 0x9bc0c05b, 
	0x00000000, 0x75b7b7c2, 0x00000000, 0xe1fdfd1c, 0x00000000, 0x3d9393ae, 0x00000000, 0x4c26266a, 
	0x00000000, 0x6c36365a, 0x00000000, 0x7e3f3f41, 0x00000000, 0xf5f7f702, 0x00000000, 0x83cccc4f, 
	0x00000000, 0x6834345c, 0x00000000, 0x51a5a5f4, 0x00000000, 0xd1e5e534, 0x00000000, 0xf9f1f108, 
	0x00000000, 0xe2717193, 0x00000000, 0xabd8d873, 0x00000000, 0x62313153, 0x00000000, 0x2a15153f, 
	0x00000000, 0x0804040c, 0x00000000, 0x95c7c752, 0x00000000, 0x46232365, 0x00000000, 0x9dc3c35e, 
	0x00000000, 0x30181828, 0x00000000, 0x379696a1, 0x00000000, 0x0a05050f, 0x00000000, 0x2f9a9ab5, 
	0x00000000, 0x0e070709, 0x00000000, 0x24121236, 0x00000000, 0x1b80809b, 0x00000000, 0xdfe2e23d, 
	0x00000000, 0xcdebeb26, 0x00000000, 0x4e272769, 0x00000000, 0x7fb2b2cd, 0x00000000, 0xea75759f, 
	0x00000000, 0x1209091b, 0x00000000, 0x1d83839e, 0x00000000, 0x582c2c74, 0x00000000, 0x341a1a2e, 
	0x00000000, 0x361b1b2d, 0x00000000, 0xdc6e6eb2, 0x00000000, 0xb45a5aee, 0x00000000, 0x5ba0a0fb, 
	0x00000000, 0xa45252f6, 0x00000000, 0x763b3b4d, 0x00000000, 0xb7d6d661, 0x00000000, 0x7db3b3ce, 
	0x00000000, 0x5229297b, 0x00000000, 0xdde3e33e, 0x00000000, 0x5e2f2f71, 0x00000000, 0x13848497, 
	0x00000000, 0xa65353f5, 0x00000000, 0xb9d1d168, 0x00000000, 0x00000000, 0x00000000, 0xc1eded2c, 
	0x00000000, 0x40202060, 0x00000000, 0xe3fcfc1f, 0x00000000, 0x79b1b1c8, 0x00000000, 0xb65b5bed, 
	0x00000000, 0xd46a6abe, 0x00000000, 0x8dcbcb46, 0x00000000, 0x67bebed9, 0x00000000, 0x7239394b, 
	0x00000000, 0x944a4ade, 0x00000000, 0x984c4cd4, 0x00000000, 0xb05858e8, 0x00000000, 0x85cfcf4a, 
	0x00000000, 0xbbd0d06b, 0x00000000, 0xc5efef2a, 0x00000000, 0x4faaaae5, 0x00000000, 0xedfbfb16, 
	0x00000000, 0x864343c5, 0x00000000, 0x9a4d4dd7, 0x00000000, 0x66333355, 0x00000000, 0x11858594, 
	0x00000000, 0x8a4545cf, 0x00000000, 0xe9f9f910, 0x00000000, 0x04020206, 0x00000000, 0xfe7f7f81, 
	0x00000000, 0xa05050f0, 0x00000000, 0x783c3c44, 0x00000000, 0x259f9fba, 0x00000000, 0x4ba8a8e3, 
	0x00000000, 0xa25151f3, 0x00000000, 0x5da3a3fe, 0x00000000, 0x804040c0, 0x00000000, 0x058f8f8a, 
	0x00000000, 0x3f9292ad, 0x00000000, 0x219d9dbc, 0x00000000, 0x70383848, 0x00000000, 0xf1f5f504, 
	0x00000000, 0x63bcbcdf, 0x00000000, 0x77b6b6c1, 0x00000000, 0xafdada75, 0x00000000, 0x42212163, 
	0x00000000, 0x20101030, 0x00000000, 0xe5ffff1a, 0x00000000, 0xfdf3f30e, 0x00000000, 0xbfd2d26d, 
	0x00000000, 0x81cdcd4c, 0x00000000, 0x180c0c14, 0x00000000, 0x26131335, 0x00000000, 0xc3ecec2f, 
	0x00000000, 0xbe5f5fe1, 0x00000000, 0x359797a2, 0x00000000, 0x884444cc, 0x00000000, 0x2e171739, 
	0x00000000, 0x93c4c457, 0x00000000, 0x55a7a7f2, 0x00000000, 0xfc7e7e82, 0x00000000, 0x7a3d3d47, 
	0x00000000, 0xc86464ac, 0x00000000, 0xba5d5de7, 0x00000000, 0x3219192b, 0x00000000, 0xe6737395, 
	0x00000000, 0xc06060a0, 0x00000000, 0x19818198, 0x00000000, 0x9e4f4fd1, 0x00000000, 0xa3dcdc7f, 
	0x00000000, 0x44222266, 0x00000000, 0x542a2a7e, 0x00000000, 0x3b9090ab, 0x00000000, 0x0b888883, 
	0x00000000, 0x8c4646ca, 0x00000000, 0xc7eeee29, 0x00000000, 0x6bb8b8d3, 0x00000000, 0x2814143c, 
	0x00000000, 0xa7dede79, 0x00000000, 0xbc5e5ee2, 0x00000000, 0x160b0b1d, 0x00000000, 0xaddbdb76, 
	0x00000000, 0xdbe0e03b, 0x00000000, 0x64323256, 0x00000000, 0x743a3a4e, 0x00000000, 0x140a0a1e, 
	0x00000000, 0x924949db, 0x00000000, 0x0c06060a, 0x00000000, 0x4824246c, 0x00000000, 0xb85c5ce4, 
	0x00000000, 0x9fc2c25d, 0x00000000, 0xbdd3d36e, 0x00000000, 0x43acacef, 0x00000000, 0xc46262a6, 
	0x00000000, 0x399191a8, 0x00000000, 0x319595a4, 0x00000000, 0xd3e4e437, 0x00000000, 0xf279798b, 
	0x00000000, 0xd5e7e732, 0x00000000, 0x8bc8c843, 0x00000000, 0x6e373759, 0x00000000, 0xda6d6db7, 
	0x00000000, 0x018d8d8c, 0x00000000, 0xb1d5d564, 0x00000000, 0x9c4e4ed2, 0x00000000, 0x49a9a9e0, 
	0x00000000, 0xd86c6cb4, 0x00000000, 0xac5656fa, 0x00000000, 0xf3f4f407, 0x00000000, 0xcfeaea25, 
	0x00000000, 0xca6565af, 0x00000000, 0xf47a7a8e, 0x00000000, 0x47aeaee9, 0x00000000, 0x10080818, 
	0x00000000, 0x6fbabad5, 0x00000000, 0xf0787888, 0x00000000, 0x4a25256f, 0x00000000, 0x5c2e2e72, 
	0x00000000, 0x381c1c24, 0x00000000, 0x57a6a6f1, 0x00000000, 0x73b4b4c7, 0x00000000, 0x97c6c651, 
	0x00000000, 0xcbe8e823, 0x00000000, 0xa1dddd7c, 0x00000000, 0xe874749c, 0x00000000, 0x3e1f1f21, 
	0x00000000, 0x964b4bdd, 0x00000000, 0x61bdbddc, 0x00000000, 0x0d8b8b86, 0x00000000, 0x0f8a8a85, 
	0x00000000, 0xe0707090, 0x00000000, 0x7c3e3e42, 0x00000000, 0x71b5b5c4, 0x00000000, 0xcc6666aa, 
	0x00000000, 0x904848d8, 0x00000000, 0x06030305, 0x00000000, 0xf7f6f601, 0x00000000, 0x1c0e0e12, 
	0x00000000, 0xc26161a3, 0x00000000, 0x6a35355f, 0x00000000, 0xae5757f9, 0x00000000, 0x69b9b9d0, 
	0x00000000, 0x17868691, 0x00000000, 0x99c1c158, 0x00000000, 0x3a1d1d27, 0x00000000, 0x279e9eb9, 
	0x00000000, 0xd9e1e138, 0x00000000, 0xebf8f813, 0x00000000, 0x2b9898b3, 0x00000000, 0x22111133, 
	0x00000000, 0xd26969bb, 0x00000000, 0xa9d9d970, 0x00000000, 0x078e8e89, 0x00000000, 0x339494a7, 
	0x00000000, 0x2d9b9bb6, 0x00000000, 0x3c1e1e22, 0x00000000, 0x15878792, 0x00000000, 0xc9e9e920, 
	0x00000000, 0x87cece49, 0x00000000, 0xaa5555ff, 0x00000000, 0x50282878, 0x00000000, 0xa5dfdf7a, 
	0x00000000, 0x038c8c8f, 0x00000000, 0x59a1a1f8, 0x00000000, 0x09898980, 0x00000000, 0x1a0d0d17, 
	0x00000000, 0x65bfbfda, 0x00000000, 0xd7e6e631, 0x00000000, 0x844242c6, 0x00000000, 0xd06868b8, 
	0x00000000, 0x824141c3, 0x00000000, 0x299999b0, 0x00000000, 0x5a2d2d77, 0x00000000, 0x1e0f0f11, 
	0x00000000, 0x7bb0b0cb, 0x00000000, 0xa85454fc, 0x00000000, 0x6dbbbbd6, 0x00000000, 0x2c16163a
};

static const u32 TBLD1[512]={
	0x00000000, 0xa5c66363, 0x00000000, 0x84f87c7c, 0x00000000, 0x99ee7777, 0x00000000, 0x8df67b7b, 
	0x00000000, 0x0dfff2f2, 0x00000000, 0xbdd66b6b, 0x00000000, 0xb1de6f6f, 0x00000000, 0x5491c5c5, 
	0x00000000, 0x50603030, 0x00000000, 0x03020101, 0x00000000, 0xa9ce6767, 0x00000000, 0x7d562b2b, 
	0x00000000, 0x19e7fefe, 0x00000000, 0x62b5d7d7, 0x00000000, 0xe64dabab, 0x00000000, 0x9aec7676, 
	0x00000000, 0x458fcaca, 0x00000000, 0x9d1f8282, 0x00000000, 0x4089c9c9, 0x00000000, 0x87fa7d7d, 
	0x00000000, 0x15effafa, 0x00000000, 0xebb25959, 0x00000000, 0xc98e4747, 0x00000000, 0x0bfbf0f0, 
	0x00000000, 0xec41adad, 0x00000000, 0x67b3d4d4, 0x00000000, 0xfd5fa2a2, 0x00000000, 0xea45afaf, 
	0x00000000, 0xbf239c9c, 0x00000000, 0xf753a4a4, 0x00000000, 0x96e47272, 0x00000000, 0x5b9bc0c0, 
	0x00000000, 0xc275b7b7, 0x00000000, 0x1ce1fdfd, 0x00000000, 0xae3d9393, 0x00000000, 0x6a4c2626, 
	0x00000000, 0x5a6c3636, 0x00000000, 0x417e3f3f, 0x00000000, 0x02f5f7f7, 0x00000000, 0x4f83cccc, 
	0x00000000, 0x5c683434, 0x00000000, 0xf451a5a5, 0x00000000, 0x34d1e5e5, 0x00000000, 0x08f9f1f1, 
	0x00000000, 0x93e27171, 0x00000000, 0x73abd8d8, 0x00000000, 0x53623131, 0x00000000, 0x3f2a1515, 
	0x00000000, 0x0c080404, 0x00000000, 0x5295c7c7, 0x00000000, 0x65462323, 0x00000000, 0x5e9dc3c3, 
	0x00000000, 0x28301818, 0x00000000, 0xa1379696, 0x00000000, 0x0f0a0505, 0x00000000, 0xb52f9a9a, 
	0x00000000, 0x090e0707, 0x00000000, 0x36241212, 0x00000000, 0x9b1b8080, 0x00000000, 0x3ddfe2e2, 
	0x00000000, 0x26cdebeb, 0x00000000, 0x694e2727, 0x00000000, 0xcd7fb2b2, 0x00000000, 0x9fea7575, 
	0x00000000, 0x1b120909, 0x00000000, 0x9e1d8383, 0x00000000, 0x74582c2c, 0x00000000, 0x2e341a1a, 
	0x00000000, 0x2d361b1b, 0x00000000, 0xb2dc6e6e, 0x00000000, 0xeeb45a5a, 0x00000000, 0xfb5ba0a0, 
	0x00000000, 0xf6a45252, 0x00000000, 0x4d763b3b, 0x00000000, 0x61b7d6d6, 0x00000000, 0xce7db3b3, 
	0x00000000, 0x7b522929, 0x00000000, 0x3edde3e3, 0x00000000, 0x715e2f2f, 0x00000000, 0x97138484, 
	0x00000000, 0xf5a65353, 0x00000000, 0x68b9d1d1, 0x00000000, 0x00000000, 0x00000000, 0x2cc1eded, 
	0x00000000, 0x60402020, 0x00000000, 0x1fe3fcfc, 0x00000000, 0xc879b1b1, 0x00000000, 0xedb65b5b, 
	0x00000000, 0xbed46a6a, 0x00000000, 0x468dcbcb, 0x00000000, 0xd967bebe, 0x00000000, 0x4b723939, 
	0x00000000, 0xde944a4a, 0x00000000, 0xd4984c4c, 0x00000000, 0xe8b05858, 0x00000000, 0x4a85cfcf, 
	0x00000000, 0x6bbbd0d0, 0x00000000, 0x2ac5efef, 0x00000000, 0xe54faaaa, 0x00000000, 0x16edfbfb, 
	0x00000000, 0xc5864343, 0x00000000, 0xd79a4d4d, 0x00000000, 0x55663333, 0x00000000, 0x94118585, 
	0x00000000, 0xcf8a4545, 0x00000000, 0x10e9f9f9, 0x00000000, 0x06040202, 0x00000000, 0x81fe7f7f, 
	0x00000000, 0xf0a05050, 0x00000000, 0x44783c3c, 0x00000000, 0xba259f9f, 0x00000000, 0xe34ba8a8, 
	0x00000000, 0xf3a25151, 0x00000000, 0xfe5da3a3, 0x00000000, 0xc0804040, 0x00000000, 0x8a058f8f, 
	0x00000000, 0xad3f9292, 0x00000000, 0xbc219d9d, 0x00000000, 0x48703838, 0x00000000, 0x04f1f5f5, 
	0x00000000, 0xdf63bcbc, 0x00000000, 0xc177b6b6, 0x00000000, 0x75afdada, 0x00000000, 0x63422121, 
	0x00000000, 0x30201010, 0x00000000, 0x1ae5ffff, 0x00000000, 0x0efdf3f3, 0x00000000, 0x6dbfd2d2, 
	0x00000000, 0x4c81cdcd, 0x00000000, 0x14180c0c, 0x00000000, 0x35261313, 0x00000000, 0x2fc3ecec, 
	0x00000000, 0xe1be5f5f, 0x00000000, 0xa2359797, 0x00000000, 0xcc884444, 0x00000000, 0x392e1717, 
	0x00000000, 0x5793c4c4, 0x00000000, 0xf255a7a7, 0x00000000, 0x82fc7e7e, 0x00000000, 0x477a3d3d, 
	0x00000000, 0xacc86464, 0x00000000, 0xe7ba5d5d, 0x00000000, 0x2b321919, 0x00000000, 0x95e67373, 
	0x00000000, 0xa0c06060, 0x00000000, 0x98198181, 0x00000000, 0xd19e4f4f, 0x00000000, 0x7fa3dcdc, 
	0x00000000, 0x66442222, 0x00000000, 0x7e542a2a, 0x00000000, 0xab3b9090, 0x00000000, 0x830b8888, 
	0x00000000, 0xca8c4646, 0x00000000, 0x29c7eeee, 0x00000000, 0xd36bb8b8, 0x00000000, 0x3c281414, 
	0x00000000, 0x79a7dede, 0x00000000, 0xe2bc5e5e, 0x00000000, 0x1d160b0b, 0x00000000, 0x76addbdb, 
	0x00000000, 0x3bdbe0e0, 0x00000000, 0x56643232, 0x00000000, 0x4e743a3a, 0x00000000, 0x1e140a0a, 
	0x00000000, 0xdb924949, 0x00000000, 0x0a0c0606, 0x00000000, 0x6c482424, 0x00000000, 0xe4b85c5c, 
	0x00000000, 0x5d9fc2c2, 0x00000000, 0x6ebdd3d3, 0x00000000, 0xef43acac, 0x00000000, 0xa6c46262, 
	0x00000000, 0xa8399191, 0x00000000, 0xa4319595, 0x00000000, 0x37d3e4e4, 0x00000000, 0x8bf27979, 
	0x00000000, 0x32d5e7e7, 0x00000000, 0x438bc8c8, 0x00000000, 0x596e3737, 0x00000000, 0xb7da6d6d, 
	0x00000000, 0x8c018d8d, 0x00000000, 0x64b1d5d5, 0x00000000, 0xd29c4e4e, 0x00000000, 0xe049a9a9, 
	0x00000000, 0xb4d86c6c, 0x00000000, 0xfaac5656, 0x00000000, 0x07f3f4f4, 0x00000000, 0x25cfeaea, 
	0x00000000, 0xafca6565, 0x00000000, 0x8ef47a7a, 0x00000000, 0xe947aeae, 0x00000000, 0x18100808, 
	0x00000000, 0xd56fbaba, 0x00000000, 0x88f07878, 0x00000000, 0x6f4a2525, 0x00000000, 0x725c2e2e, 
	0x00000000, 0x24381c1c, 0x00000000, 0xf157a6a6, 0x00000000, 0xc773b4b4, 0x00000000, 0x5197c6c6, 
	0x00000000, 0x23cbe8e8, 0x00000000, 0x7ca1dddd, 0x00000000, 0x9ce87474, 0x00000000, 0x213e1f1f, 
	0x00000000, 0xdd964b4b, 0x00000000, 0xdc61bdbd, 0x00000000, 0x860d8b8b, 0x00000000, 0x850f8a8a, 
	0x00000000, 0x90e07070, 0x00000000, 0x427c3e3e, 0x00000000, 0xc471b5b5, 0x00000000, 0xaacc6666, 
	0x00000000, 0xd8904848, 0x00000000, 0x05060303, 0x00000000, 0x01f7f6f6, 0x00000000, 0x121c0e0e, 
	0x00000000, 0xa3c26161, 0x00000000, 0x5f6a3535, 0x00000000, 0xf9ae5757, 0x00000000, 0xd069b9b9, 
	0x00000000, 0x91178686, 0x00000000, 0x5899c1c1, 0x00000000, 0x273a1d1d, 0x00000000, 0xb9279e9e, 
	0x00000000, 0x38d9e1e1, 0x00000000, 0x13ebf8f8, 0x00000000, 0xb32b9898, 0x00000000, 0x33221111, 
	0x00000000, 0xbbd26969, 0x00000000, 0x70a9d9d9, 0x00000000, 0x89078e8e, 0x00000000, 0xa7339494, 
	0x00000000, 0xb62d9b9b, 0x00000000, 0x223c1e1e, 0x00000000, 0x92158787, 0x00000000, 0x20c9e9e9, 
	0x00000000, 0x4987cece, 0x00000000, 0xffaa5555, 0x00000000, 0x78502828, 0x00000000, 0x7aa5dfdf, 
	0x00000000, 0x8f038c8c, 0x00000000, 0xf859a1a1, 0x00000000, 0x80098989, 0x00000000, 0x171a0d0d, 
	0x00000000, 0xda65bfbf, 0x00000000, 0x31d7e6e6, 0x00000000, 0xc6844242, 0x00000000, 0xb8d06868, 
	0x00000000, 0xc3824141, 0x00000000, 0xb0299999, 0x00000000, 0x775a2d2d, 0x00000000, 0x111e0f0f, 
	0x00000000, 0xcb7bb0b0, 0x00000000, 0xfca85454, 0x00000000, 0xd66dbbbb, 0x00000000, 0x3a2c1616
};

static const u32 TBLD2[512]={
	0x00000000, 0x63a5c663, 0x00000000, 0x7c84f87c, 0x00000000, 0x7799ee77, 0x00000000, 0x7b8df67b, 
	0x00000000, 0xf20dfff2, 0x00000000, 0x6bbdd66b, 0x00000000, 0x6fb1de6f, 0x00000000, 0xc55491c5, 
	0x00000000, 0x30506030, 0x00000000, 0x01030201, 0x00000000, 0x67a9ce67, 0x00000000, 0x2b7d562b, 
	0x00000000, 0xfe19e7fe, 0x00000000, 0xd762b5d7, 0x00000000, 0xabe64dab, 0x00000000, 0x769aec76, 
	0x00000000, 0xca458fca, 0x00000000, 0x829d1f82, 0x00000000, 0xc94089c9, 0x00000000, 0x7d87fa7d, 
	0x00000000, 0xfa15effa, 0x00000000, 0x59ebb259, 0x00000000, 0x47c98e47, 0x00000000, 0xf00bfbf0, 
	0x00000000, 0xadec41ad, 0x00000000, 0xd467b3d4, 0x00000000, 0xa2fd5fa2, 0x00000000, 0xafea45af, 
	0x00000000, 0x9cbf239c, 0x00000000, 0xa4f753a4, 0x00000000, 0x7296e472, 0x00000000, 0xc05b9bc0, 
	0x00000000, 0xb7c275b7, 0x00000000, 0xfd1ce1fd, 0x00000000, 0x93ae3d93, 0x00000000, 0x266a4c26, 
	0x00000000, 0x365a6c36, 0x00000000, 0x3f417e3f, 0x00000000, 0xf702f5f7, 0x00000000, 0xcc4f83cc, 
	0x00000000, 0x345c6834, 0x00000000, 0xa5f451a5, 0x00000000, 0xe534d1e5, 0x00000000, 0xf108f9f1, 
	0x00000000, 0x7193e271, 0x00000000, 0xd873abd8, 0x00000000, 0x31536231, 0x00000000, 0x153f2a15, 
	0x00000000, 0x040c0804, 0x00000000, 0xc75295c7, 0x00000000, 0x23654623, 0x00000000, 0xc35e9dc3, 
	0x00000000, 0x18283018, 0x00000000, 0x96a13796, 0x00000000, 0x050f0a05, 0x00000000, 0x9ab52f9a, 
	0x00000000, 0x07090e07, 0x00000000, 0x12362412, 0x00000000, 0x809b1b80, 0x00000000, 0xe23ddfe2, 
	0x00000000, 0xeb26cdeb, 0x00000000, 0x27694e27, 0x00000000, 0xb2cd7fb2, 0x00000000, 0x759fea75, 
	0x00000000, 0x091b1209, 0x00000000, 0x839e1d83, 0x00000000, 0x2c74582c, 0x00000000, 0x1a2e341a, 
	0x00000000, 0x1b2d361b, 0x00000000, 0x6eb2dc6e, 0x00000000, 0x5aeeb45a, 0x00000000, 0xa0fb5ba0, 
	0x00000000, 0x52f6a452, 0x00000000, 0x3b4d763b, 0x00000000, 0xd661b7d6, 0x00000000, 0xb3ce7db3, 
	0x00000000, 0x297b5229, 0x00000000, 0xe33edde3, 0x00000000, 0x2f715e2f, 0x00000000, 0x84971384, 
	0x00000000, 0x53f5a653, 0x00000000, 0xd168b9d1, 0x00000000, 0x00000000, 0x00000000, 0xed2cc1ed, 
	0x00000000, 0x20604020, 0x00000000, 0xfc1fe3fc, 0x00000000, 0xb1c879b1, 0x00000000, 0x5bedb65b, 
	0x00000000, 0x6abed46a, 0x00000000, 0xcb468dcb, 0x00000000, 0xbed967be, 0x00000000, 0x394b7239, 
	0x00000000, 0x4ade944a, 0x00000000, 0x4cd4984c, 0x00000000, 0x58e8b058, 0x00000000, 0xcf4a85cf, 
	0x00000000, 0xd06bbbd0, 0x00000000, 0xef2ac5ef, 0x00000000, 0xaae54faa, 0x00000000, 0xfb16edfb, 
	0x00000000, 0x43c58643, 0x00000000, 0x4dd79a4d, 0x00000000, 0x33556633, 0x00000000, 0x85941185, 
	0x00000000, 0x45cf8a45, 0x00000000, 0xf910e9f9, 0x00000000, 0x02060402, 0x00000000, 0x7f81fe7f, 
	0x00000000, 0x50f0a050, 0x00000000, 0x3c44783c, 0x00000000, 0x9fba259f, 0x00000000, 0xa8e34ba8, 
	0x00000000, 0x51f3a251, 0x00000000, 0xa3fe5da3, 0x00000000, 0x40c08040, 0x00000000, 0x8f8a058f, 
	0x00000000, 0x92ad3f92, 0x00000000, 0x9dbc219d, 0x00000000, 0x38487038, 0x00000000, 0xf504f1f5, 
	0x00000000, 0xbcdf63bc, 0x00000000, 0xb6c177b6, 0x00000000, 0xda75afda, 0x00000000, 0x21634221, 
	0x00000000, 0x10302010, 0x00000000, 0xff1ae5ff, 0x00000000, 0xf30efdf3, 0x00000000, 0xd26dbfd2, 
	0x00000000, 0xcd4c81cd, 0x00000000, 0x0c14180c, 0x00000000, 0x13352613, 0x00000000, 0xec2fc3ec, 
	0x00000000, 0x5fe1be5f, 0x00000000, 0x97a23597, 0x00000000, 0x44cc8844, 0x00000000, 0x17392e17, 
	0x00000000, 0xc45793c4, 0x00000000, 0xa7f255a7, 0x00000000, 0x7e82fc7e, 0x00000000, 0x3d477a3d, 
	0x00000000, 0x64acc864, 0x00000000, 0x5de7ba5d, 0x00000000, 0x192b3219, 0x00000000, 0x7395e673, 
	0x00000000, 0x60a0c060, 0x00000000, 0x81981981, 0x00000000, 0x4fd19e4f, 0x00000000, 0xdc7fa3dc, 
	0x00000000, 0x22664422, 0x00000000, 0x2a7e542a, 0x00000000, 0x90ab3b90, 0x00000000, 0x88830b88, 
	0x00000000, 0x46ca8c46, 0x00000000, 0xee29c7ee, 0x00000000, 0xb8d36bb8, 0x00000000, 0x143c2814, 
	0x00000000, 0xde79a7de, 0x00000000, 0x5ee2bc5e, 0x00000000, 0x0b1d160b, 0x00000000, 0xdb76addb, 
	0x00000000, 0xe03bdbe0, 0x00000000, 0x32566432, 0x00000000, 0x3a4e743a, 0x00000000, 0x0a1e140a, 
	0x00000000, 0x49db9249, 0x00000000, 0x060a0c06, 0x00000000, 0x246c4824, 0x00000000, 0x5ce4b85c, 
	0x00000000, 0xc25d9fc2, 0x00000000, 0xd36ebdd3, 0x00000000, 0xacef43ac, 0x00000000, 0x62a6c462, 
	0x00000000, 0x91a83991, 0x00000000, 0x95a43195, 0x00000000, 0xe437d3e4, 0x00000000, 0x798bf279, 
	0x00000000, 0xe732d5e7, 0x00000000, 0xc8438bc8, 0x00000000, 0x37596e37, 0x00000000, 0x6db7da6d, 
	0x00000000, 0x8d8c018d, 0x00000000, 0xd564b1d5, 0x00000000, 0x4ed29c4e, 0x00000000, 0xa9e049a9, 
	0x00000000, 0x6cb4d86c, 0x00000000, 0x56faac56, 0x00000000, 0xf407f3f4, 0x00000000, 0xea25cfea, 
	0x00000000, 0x65afca65, 0x00000000, 0x7a8ef47a, 0x00000000, 0xaee947ae, 0x00000000, 0x08181008, 
	0x00000000, 0xbad56fba, 0x00000000, 0x7888f078, 0x00000000, 0x256f4a25, 0x00000000, 0x2e725c2e, 
	0x00000000, 0x1c24381c, 0x00000000, 0xa6f157a6, 0x00000000, 0xb4c773b4, 0x00000000, 0xc65197c6, 
	0x00000000, 0xe823cbe8, 0x00000000, 0xdd7ca1dd, 0x00000000, 0x749ce874, 0x00000000, 0x1f213e1f, 
	0x00000000, 0x4bdd964b, 0x00000000, 0xbddc61bd, 0x00000000, 0x8b860d8b, 0x00000000, 0x8a850f8a, 
	0x00000000, 0x7090e070, 0x00000000, 0x3e427c3e, 0x00000000, 0xb5c471b5, 0x00000000, 0x66aacc66, 
	0x00000000, 0x48d89048, 0x00000000, 0x03050603, 0x00000000, 0xf601f7f6, 0x00000000, 0x0e121c0e, 
	0x00000000, 0x61a3c261, 0x00000000, 0x355f6a35, 0x00000000, 0x57f9ae57, 0x00000000, 0xb9d069b9, 
	0x00000000, 0x86911786, 0x00000000, 0xc15899c1, 0x00000000, 0x1d273a1d, 0x00000000, 0x9eb9279e, 
	0x00000000, 0xe138d9e1, 0x00000000, 0xf813ebf8, 0x00000000, 0x98b32b98, 0x00000000, 0x11332211, 
	0x00000000, 0x69bbd269, 0x00000000, 0xd970a9d9, 0x00000000, 0x8e89078e, 0x00000000, 0x94a73394, 
	0x00000000, 0x9bb62d9b, 0x00000000, 0x1e223c1e, 0x00000000, 0x87921587, 0x00000000, 0xe920c9e9, 
	0x00000000, 0xce4987ce, 0x00000000, 0x55ffaa55, 0x00000000, 0x28785028, 0x00000000, 0xdf7aa5df, 
	0x00000000, 0x8c8f038c, 0x00000000, 0xa1f859a1, 0x00000000, 0x89800989, 0x00000000, 0x0d171a0d, 
	0x00000000, 0xbfda65bf, 0x00000000, 0xe631d7e6, 0x00000000, 0x42c68442, 0x00000000, 0x68b8d068, 
	0x00000000, 0x41c38241, 0x00000000, 0x99b02999, 0x00000000, 0x2d775a2d, 0x00000000, 0x0f111e0f, 
	0x00000000, 0xb0cb7bb0, 0x00000000, 0x54fca854, 0x00000000, 0xbbd66dbb, 0x00000000, 0x163a2c16
};

static const u32 TBLD3[512]={
	0x00000000, 0x6363a5c6, 0x00000000, 0x7c7c84f8, 0x00000000, 0x777799ee, 0x00000000, 0x7b7b8df6, 
	0x00000000, 0xf2f20dff, 0x00000000, 0x6b6bbdd6, 0x00000000, 0x6f6fb1de, 0x00000000, 0xc5c55491, 
	0x00000000, 0x30305060, 0x00000000, 0x01010302, 0x00000000, 0x6767a9ce, 0x00000000, 0x2b2b7d56, 
	0x00000000, 0xfefe19e7, 0x00000000, 0xd7d762b5, 0x00000000, 0xababe64d, 0x00000000, 0x76769aec, 
	0x00000000, 0xcaca458f, 0x00000000, 0x82829d1f, 0x00000000, 0xc9c94089, 0x00000000, 0x7d7d87fa, 
	0x00000000, 0xfafa15ef, 0x00000000, 0x5959ebb2, 0x00000000, 0x4747c98e, 0x00000000, 0xf0f00bfb, 
	0x00000000, 0xadadec41, 0x00000000, 0xd4d467b3, 0x00000000, 0xa2a2fd5f, 0x00000000, 0xafafea45, 
	0x00000000, 0x9c9cbf23, 0x00000000, 0xa4a4f753, 0x00000000, 0x727296e4, 0x00000000, 0xc0c05b9b, 
	0x00000000, 0xb7b7c275, 0x00000000, 0xfdfd1ce1, 0x00000000, 0x9393ae3d, 0x00000000, 0x26266a4c, 
	0x00000000, 0x36365a6c, 0x00000000, 0x3f3f417e, 0x00000000, 0xf7f702f5, 0x00000000, 0xcccc4f83, 
	0x00000000, 0x34345c68, 0x00000000, 0xa5a5f451, 0x00000000, 0xe5e534d1, 0x00000000, 0xf1f108f9, 
	0x00000000, 0x717193e2, 0x00000000, 0xd8d873ab, 0x00000000, 0x31315362, 0x00000000, 0x15153f2a, 
	0x00000000, 0x04040c08, 0x00000000, 0xc7c75295, 0x00000000, 0x23236546, 0x00000000, 0xc3c35e9d, 
	0x00000000, 0x18182830, 0x00000000, 0x9696a137, 0x00000000, 0x05050f0a, 0x00000000, 0x9a9ab52f, 
	0x00000000, 0x0707090e, 0x00000000, 0x12123624, 0x00000000, 0x80809b1b, 0x00000000, 0xe2e23ddf, 
	0x00000000, 0xebeb26cd, 0x00000000, 0x2727694e, 0x00000000, 0xb2b2cd7f, 0x00000000, 0x75759fea, 
	0x00000000, 0x09091b12, 0x00000000, 0x83839e1d, 0x00000000, 0x2c2c7458, 0x00000000, 0x1a1a2e34, 
	0x00000000, 0x1b1b2d36, 0x00000000, 0x6e6eb2dc, 0x00000000, 0x5a5aeeb4, 0x00000000, 0xa0a0fb5b, 
	0x00000000, 0x5252f6a4, 0x00000000, 0x3b3b4d76, 0x00000000, 0xd6d661b7, 0x00000000, 0xb3b3ce7d, 
	0x00000000, 0x29297b52, 0x00000000, 0xe3e33edd, 0x00000000, 0x2f2f715e, 0x00000000, 0x84849713, 
	0x00000000, 0x5353f5a6, 0x00000000, 0xd1d168b9, 0x00000000, 0x00000000, 0x00000000, 0xeded2cc1, 
	0x00000000, 0x20206040, 0x00000000, 0xfcfc1fe3, 0x00000000, 0xb1b1c879, 0x00000000, 0x5b5bedb6, 
	0x00000000, 0x6a6abed4, 0x00000000, 0xcbcb468d, 0x00000000, 0xbebed967, 0x00000000, 0x39394b72, 
	0x00000000, 0x4a4ade94, 0x00000000, 0x4c4cd498, 0x00000000, 0x5858e8b0, 0x00000000, 0xcfcf4a85, 
	0x00000000, 0xd0d06bbb, 0x00000000, 0xefef2ac5, 0x00000000, 0xaaaae54f, 0x00000000, 0xfbfb16ed, 
	0x00000000, 0x4343c586, 0x00000000, 0x4d4dd79a, 0x00000000, 0x33335566, 0x00000000, 0x85859411, 
	0x00000000, 0x4545cf8a, 0x00000000, 0xf9f910e9, 0x00000000, 0x02020604, 0x00000000, 0x7f7f81fe, 
	0x00000000, 0x5050f0a0, 0x00000000, 0x3c3c4478, 0x00000000, 0x9f9fba25, 0x00000000, 0xa8a8e34b, 
	0x00000000, 0x5151f3a2, 0x00000000, 0xa3a3fe5d, 0x00000000, 0x4040c080, 0x00000000, 0x8f8f8a05, 
	0x00000000, 0x9292ad3f, 0x00000000, 0x9d9dbc21, 0x00000000, 0x38384870, 0x00000000, 0xf5f504f1, 
	0x00000000, 0xbcbcdf63, 0x00000000, 0xb6b6c177, 0x00000000, 0xdada75af, 0x00000000, 0x21216342, 
	0x00000000, 0x10103020, 0x00000000, 0xffff1ae5, 0x00000000, 0xf3f30efd, 0x00000000, 0xd2d26dbf, 
	0x00000000, 0xcdcd4c81, 0x00000000, 0x0c0c1418, 0x00000000, 0x13133526, 0x00000000, 0xecec2fc3, 
	0x00000000, 0x5f5fe1be, 0x00000000, 0x9797a235, 0x00000000, 0x4444cc88, 0x00000000, 0x1717392e, 
	0x00000000, 0xc4c45793, 0x00000000, 0xa7a7f255, 0x00000000, 0x7e7e82fc, 0x00000000, 0x3d3d477a, 
	0x00000000, 0x6464acc8, 0x00000000, 0x5d5de7ba, 0x00000000, 0x19192b32, 0x00000000, 0x737395e6, 
	0x00000000, 0x6060a0c0, 0x00000000, 0x81819819, 0x00000000, 0x4f4fd19e, 0x00000000, 0xdcdc7fa3, 
	0x00000000, 0x22226644, 0x00000000, 0x2a2a7e54, 0x00000000, 0x9090ab3b, 0x00000000, 0x8888830b, 
	0x00000000, 0x4646ca8c, 0x00000000, 0xeeee29c7, 0x00000000, 0xb8b8d36b, 0x00000000, 0x14143c28, 
	0x00000000, 0xdede79a7, 0x00000000, 0x5e5ee2bc, 0x00000000, 0x0b0b1d16, 0x00000000, 0xdbdb76ad, 
	0x00000000, 0xe0e03bdb, 0x00000000, 0x32325664, 0x00000000, 0x3a3a4e74, 0x00000000, 0x0a0a1e14, 
	0x00000000, 0x4949db92, 0x00000000, 0x06060a0c, 0x00000000, 0x24246c48, 0x00000000, 0x5c5ce4b8, 
	0x00000000, 0xc2c25d9f, 0x00000000, 0xd3d36ebd, 0x00000000, 0xacacef43, 0x00000000, 0x6262a6c4, 
	0x00000000, 0x9191a839, 0x00000000, 0x9595a431, 0x00000000, 0xe4e437d3, 0x00000000, 0x79798bf2, 
	0x00000000, 0xe7e732d5, 0x00000000, 0xc8c8438b, 0x00000000, 0x3737596e, 0x00000000, 0x6d6db7da, 
	0x00000000, 0x8d8d8c01, 0x00000000, 0xd5d564b1, 0x00000000, 0x4e4ed29c, 0x00000000, 0xa9a9e049, 
	0x00000000, 0x6c6cb4d8, 0x00000000, 0x5656faac, 0x00000000, 0xf4f407f3, 0x00000000, 0xeaea25cf, 
	0x00000000, 0x6565afca, 0x00000000, 0x7a7a8ef4, 0x00000000, 0xaeaee947, 0x00000000, 0x08081810, 
	0x00000000, 0xbabad56f, 0x00000000, 0x787888f0, 0x00000000, 0x25256f4a, 0x00000000, 0x2e2e725c, 
	0x00000000, 0x1c1c2438, 0x00000000, 0xa6a6f157, 0x00000000, 0xb4b4c773, 0x00000000, 0xc6c65197, 
	0x00000000, 0xe8e823cb, 0x00000000, 0xdddd7ca1, 0x00000000, 0x74749ce8, 0x00000000, 0x1f1f213e, 
	0x00000000, 0x4b4bdd96, 0x00000000, 0xbdbddc61, 0x00000000, 0x8b8b860d, 0x00000000, 0x8a8a850f, 
	0x00000000, 0x707090e0, 0x00000000, 0x3e3e427c, 0x00000000, 0xb5b5c471, 0x00000000, 0x6666aacc, 
	0x00000000, 0x4848d890, 0x00000000, 0x03030506, 0x00000000, 0xf6f601f7, 0x00000000, 0x0e0e121c, 
	0x00000000, 0x6161a3c2, 0x00000000, 0x35355f6a, 0x00000000, 0x5757f9ae, 0x00000000, 0xb9b9d069, 
	0x00000000, 0x86869117, 0x00000000, 0xc1c15899, 0x00000000, 0x1d1d273a, 0x00000000, 0x9e9eb927, 
	0x00000000, 0xe1e138d9, 0x00000000, 0xf8f813eb, 0x00000000, 0x9898b32b, 0x00000000, 0x11113322, 
	0x00000000, 0x6969bbd2, 0x00000000, 0xd9d970a9, 0x00000000, 0x8e8e8907, 0x00000000, 0x9494a733, 
	0x00000000, 0x9b9bb62d, 0x00000000, 0x1e1e223c, 0x00000000, 0x87879215, 0x00000000, 0xe9e920c9, 
	0x00000000, 0xcece4987, 0x00000000, 0x5555ffaa, 0x00000000, 0x28287850, 0x00000000, 0xdfdf7aa5, 
	0x00000000, 0x8c8c8f03, 0x00000000, 0xa1a1f859, 0x00000000, 0x89898009, 0x00000000, 0x0d0d171a, 
	0x00000000, 0xbfbfda65, 0x00000000, 0xe6e631d7, 0x00000000, 0x4242c684, 0x00000000, 0x6868b8d0, 
	0x00000000, 0x4141c382, 0x00000000, 0x9999b029, 0x00000000, 0x2d2d775a, 0x00000000, 0x0f0f111e, 
	0x00000000, 0xb0b0cb7b, 0x00000000, 0x5454fca8, 0x00000000, 0xbbbbd66d, 0x00000000, 0x16163a2c
};

static const u8 CNS[32][2] = {
    {0x01, 0x00}, {0x03, 0x02}, 
    {0x05, 0x04}, {0x07, 0x06},
    {0x09, 0x08}, {0x0b, 0x0a}, 
    {0x0d, 0x0c}, {0x0f, 0x0e},
    {0x11, 0x10}, {0x13, 0x12}, 
    {0x15, 0x14}, {0x17, 0x16},
    {0x19, 0x18}, {0x1b, 0x1a}, 
    {0x1d, 0x1c}, {0x1f, 0x1e},
    {0x21, 0x20}, {0x23, 0x22}, 
    {0x25, 0x24}, {0x27, 0x26},
    {0x29, 0x28}, {0x2b, 0x2a}, 
    {0x2d, 0x2c}, {0x2f, 0x2e},
    {0x31, 0x30}, {0x33, 0x32}, 
    {0x35, 0x34}, {0x37, 0x36},
    {0x39, 0x38}, {0x3b, 0x3a}, 
    {0x3d, 0x3c}, {0x3f, 0x3e}
};

HashReturn Init256(hashState *state)
{
	ST256(state).bitlen[0] = 0;

	ST256(state).rembitlen = 0;

	memset(ST256(state).buffer, 0, sizeof(ST256(state).buffer));
	memcpy(ST256(state).hash, H_init256, sizeof(ST256(state).hash));

	return SUCCESS;
}

HashReturn Update256(hashState *state, const BitSequence *data, Datalength databitlen)
{
	u8 *p = (u8*)ST256(state).buffer;
	u32 cpylen;
	u32 len;

	ST256(state).bitlen[0] += databitlen;

	if (ST256(state).rembitlen + databitlen >= LESAMNTA256_BLOCK_BIT_LEN) {
		cpylen = LESAMNTA256_BLOCK_LEN - (ST256(state).rembitlen >> 3);

		memcpy(p + (ST256(state).rembitlen >> 3), data, cpylen);

		BYTES_SWAP256(ST256(state).buffer);

		Compression256(ST256(state).hash, ST256(state).buffer);

		databitlen -= (cpylen << 3);
		data += cpylen;
		ST256(state).rembitlen = 0;

		while (databitlen >= LESAMNTA256_BLOCK_BIT_LEN) {
			memcpy(p, data, LESAMNTA256_BLOCK_LEN);

			BYTES_SWAP256(ST256(state).buffer);

			Compression256(ST256(state).hash, ST256(state).buffer);

			databitlen -= LESAMNTA256_BLOCK_BIT_LEN;
			data += LESAMNTA256_BLOCK_LEN;
		}
	}

	/* All remaining data copy to buffer */
	if (databitlen) {
		len = databitlen >> 3;
		if (databitlen % 8 != 0) {
			len += 1;
		}

		memcpy(p + (ST256(state).rembitlen >> 3), data, len);
		ST256(state).rembitlen += databitlen;
	}

	return SUCCESS;
}

HashReturn Final256(hashState *state, BitSequence *hashval)
{
	u8 *p = (u8*)ST256(state).buffer;
	int bytePos;
	int bitPos;

	/* Get the byte position in which padding is needed */
	bytePos = ST256(state).rembitlen >> 3;

	/* Get the bit position in which last 1-bit data is stored */
	bitPos = (ST256(state).rembitlen % 8);

	if (bitPos == 0) {
		/* Concatenate with a single 1 bit and zero bits */
		p[bytePos] = 0x80;
	} else {
		/* Mask for the upper n bits (n = bitPos) */
		p[bytePos] &= (0xff << (8 - bitPos));

		/* Concatenate with a single 1 bit and zero bits */
		p[bytePos] |= (0x80 >> bitPos);
	}

	/* Concatenate with zero bits */
	memset(&p[bytePos + 1], 0, LESAMNTA256_BLOCK_LEN - (bytePos + 1));

	/* If data bit length is not a multiple of 256 bits */
	if (ST256(state).rembitlen != 0) {
		BYTES_SWAP256(ST256(state).buffer);

		Compression256(ST256(state).hash, ST256(state).buffer);

		memset(p, 0, LESAMNTA256_BLOCK_LEN);
	}

	BYTES_SWAP256(ST256(state).buffer);

	/* Store the data bit length in lower 64 bits of a 256-bit data block */
	ST256(state).buffer[6] = (u32)(ST256(state).bitlen[0] >> 32);
	ST256(state).buffer[7] = (u32)ST256(state).bitlen[0];

	Output256(ST256(state).hash, ST256(state).buffer);

	BYTES_SWAP256(ST256(state).hash);

	memcpy(hashval, &ST256(state).hash, state->hashbitlen >> 3);

	return SUCCESS;
}

HashReturn Init512(hashState *state)
{
	ST512(state).bitlen[0] = 0;
	ST512(state).bitlen[1] = 0;

	ST512(state).rembitlen = 0;

	memset(ST512(state).buffer, 0, sizeof(ST512(state).buffer));
	memcpy(ST512(state).hash, H_init512, sizeof(ST512(state).hash));

	return SUCCESS;
}

HashReturn Update512(hashState *state, const BitSequence *data, Datalength databitlen)
{
	u8 *p =(u8*) ST512(state).buffer;
	u32 cpylen;
	u64 len;

	if ((ST512(state).bitlen[1] += databitlen) < databitlen) {
		ST512(state).bitlen[0] += 1;
	}

	if ( ST512(state).rembitlen + databitlen >= LESAMNTA512_BLOCK_BIT_LEN) {
		cpylen = LESAMNTA512_BLOCK_LEN - (ST512(state).rembitlen >> 3);

		memcpy(p + (ST512(state).rembitlen >> 3), data, cpylen);

		BYTES_SWAP512(ST512(state).buffer);

		Compression512(ST512(state).hash, ST512(state).buffer);

		databitlen -= (cpylen << 3);
		data += cpylen;
		ST512(state).rembitlen = 0;

		while (databitlen >= LESAMNTA512_BLOCK_BIT_LEN) {
			memcpy(p, data, LESAMNTA512_BLOCK_LEN);

			BYTES_SWAP512(ST512(state).buffer);

			Compression512(ST512(state).hash, ST512(state).buffer);

			databitlen -= LESAMNTA512_BLOCK_BIT_LEN;
			data += LESAMNTA512_BLOCK_LEN;
		}
	}

	/* All remaining data copy to buffer */
	if (databitlen) {
		len = databitlen >> 3;
		if (databitlen % 8 != 0) {
			len += 1;
		}

		memcpy(p + (ST512(state).rembitlen >> 3), data, len);
		ST512(state).rembitlen += databitlen;
	}

	return SUCCESS;
}

HashReturn Final512(hashState *state, BitSequence *hashval)
{
	u8 *p = (u8*)ST512(state).buffer;
	int bytePos;
	int bitPos;

	/* Get the byte position in which padding is needed */
	bytePos = (ST512(state).rembitlen) >> 3;

	/* Get the bit position in which last 1-bit data is stored */
	bitPos = ST512(state).rembitlen % 8;

	if (bitPos == 0) {
		/* Concatenate with a single 1 bit and zero bits */
		p[bytePos] = 0x80;
	} else {
		/* Mask for the upper n bits (n = bitPos) */
		p[bytePos] &= (0xff << (8 - bitPos));

		/* Concatenate with a single 1 bit and zero bits */
		p[bytePos] |= (0x80 >> bitPos);
	}

	/* Concatenate with zero bits */
	memset(&p[bytePos + 1], 0, LESAMNTA512_BLOCK_LEN - (bytePos + 1));

	/* If data bit length is not a multiple of 512 bits */
	if (ST512(state).rembitlen != 0) {
		BYTES_SWAP512(ST512(state).buffer);

		Compression512(ST512(state).hash, ST512(state).buffer);

		memset(p, 0, LESAMNTA512_BLOCK_LEN);
	} 		

	BYTES_SWAP512(ST512(state).buffer);

	/* Store the data bit length in lower 128 bits of a 512-bit data block */
	ST512(state).buffer[6] = ST512(state).bitlen[0];
	ST512(state).buffer[7] = ST512(state).bitlen[1];

	Output512(ST512(state).hash, ST512(state).buffer);

	BYTES_SWAP512(ST512(state).hash);

	memcpy(hashval, &ST512(state).hash, state->hashbitlen >> 3);

	return SUCCESS;
}

HashReturn Init224(hashState *state)
{
	ST224(state).bitlen[0] = 0;

	ST224(state).rembitlen = 0;

	memset(ST224(state).buffer, 0, sizeof(ST224(state).buffer));
	memcpy(ST224(state).hash, H_init224, sizeof(ST224(state).hash));

	return SUCCESS;
}

HashReturn Update224(hashState *state, const BitSequence *data, Datalength databitlen)
{
	return  Update256(state, data, databitlen); 
}

HashReturn Final224(hashState *state, BitSequence *hashval) 
{
	return  Final256(state, hashval); 
}

HashReturn Init384(hashState *state)
{
	ST384(state).bitlen[0] = 0;
	ST384(state).bitlen[1] = 0;

	ST384(state).rembitlen = 0;

	memset(ST384(state).buffer, 0, sizeof(ST384(state).buffer));
	memcpy(ST384(state).hash, H_init384, sizeof(ST384(state).hash));

	return SUCCESS;
}

HashReturn Update384(hashState *state, const BitSequence *data, Datalength databitlen)
{
	return  Update512(state, data, databitlen); 
}

HashReturn Final384(hashState *state, BitSequence *hashval) 
{
	return  Final512(state, hashval); 
}

HashReturn Hash(int hashbitlen, const BitSequence *data, Datalength databitlen, BitSequence *hashval) 
{
	hashState state;
	HashReturn ret;

	ret = Init(&state, hashbitlen);
	if (ret != SUCCESS) {
		return ret;
	}

	ret = Update(&state, data, databitlen);
	if (ret != SUCCESS) {
		return ret;
	}

	ret = Final(&state, hashval);
	if (ret != SUCCESS) {
		return ret;
	}
	return SUCCESS;
}

HashReturn Init(hashState *state, int hashbitlen) 
{
	HashReturn ret = FAIL;

	state->hashbitlen = hashbitlen;

	switch(hashbitlen) {
		case LESAMNTA224_DIGEST_BIT_LEN:
			ret = Init224(state);
			break;
		case LESAMNTA256_DIGEST_BIT_LEN:
			ret = Init256(state);
			break;
		case LESAMNTA384_DIGEST_BIT_LEN:
			ret = Init384(state);
			break;
		case LESAMNTA512_DIGEST_BIT_LEN:
			ret = Init512(state);
			break;
		default:
			ret = BAD_HASHBITLEN;
			break;
	}

	return ret;
}

HashReturn Update(hashState *state, const BitSequence *data, Datalength databitlen) 
{
	HashReturn ret = FAIL;

	switch(state->hashbitlen) {
		case LESAMNTA224_DIGEST_BIT_LEN:
			ret = Update224(state, data, databitlen);
			break;
		case LESAMNTA256_DIGEST_BIT_LEN:
			ret = Update256(state, data, databitlen);
			break;
		case LESAMNTA384_DIGEST_BIT_LEN:
			ret = Update384(state, data, databitlen);
			break;
		case LESAMNTA512_DIGEST_BIT_LEN:
			ret = Update512(state, data, databitlen);
			break;
		default:
			ret = BAD_HASHBITLEN;
			break;
	}

	return ret;
}

HashReturn Final(hashState *state, BitSequence *hashval) 
{
	HashReturn ret = FAIL;

	switch(state->hashbitlen) {
		case LESAMNTA224_DIGEST_BIT_LEN:
			ret = Final224(state, hashval);
			break;
		case LESAMNTA256_DIGEST_BIT_LEN:
			ret = Final256(state, hashval);
			break;
		case LESAMNTA384_DIGEST_BIT_LEN:
			ret = Final384(state, hashval);
			break;
		case LESAMNTA512_DIGEST_BIT_LEN:
			ret = Final512(state, hashval);
			break;
		default:
			ret = BAD_HASHBITLEN;
			break;
	}

	return ret;
}

static void Compression256(u32 *H, const u32 *M) 
{
	int i;
	u32 midkey[8], midmsg[8];
	struct {
		u32 key[2];
		u32 msg[2];
		u32 t[2];
	} tmp;

	for (i = 0; i < 8; i++) {
		midkey[i] = H[i];
		midmsg[i] = M[i];
	}

	for (i = 0; i < 8; i++) {
		ROUND256(4, 0, 6, 4 * i);
		ROUND256(2, 6, 4, 4 * i + 1);
		ROUND256(0, 4, 2, 4 * i + 2);
		ROUND256(6, 2, 0, 4 * i + 3);
	}

	for (i = 0; i < 8; i++) {
		H[i] = M[i] ^ midmsg[i];
	}
}

static void Output256(u32 *H, const u32 *M) 
{
	int i;
	u32 midkey[8], midmsg[8];
	struct {
		u32 key[2];
		u32 msg[2];
		u32 t[2];
	} tmp;

	for (i = 0; i < 8; i++) {
		midkey[i] = H[i];
		midmsg[i] = M[i];
	}

	for (i = 0; i < 8; i++) {
		ROUND256_OUTPUT(4, 0, 6, 4 * i);
		ROUND256_OUTPUT(2, 6, 4, 4 * i + 1);
		ROUND256_OUTPUT(0, 4, 2, 4 * i + 2);
		ROUND256_OUTPUT(6, 2, 0, 4 * i + 3);
	}

	for (i = 0; i < 8; i++) {
		H[i] = M[i] ^ midmsg[i];
	}
}

static void Compression512(u64 *H, const u64 *M)
{
	int i;
	u64 midkey[8], midmsg[8];
	u64 tmpkey[2], tmpmsg[2], t[2];

	for(i = 0; i < 8; i++) {
		midkey[i] = H[i];
		midmsg[i] = M[i];
	}

	for(i = 0; i < 8; i++) {
		ROUND512(4, 0, 6, 4 * i);
		ROUND512(2, 6, 4, 4 * i + 1);
		ROUND512(0, 4, 2, 4 * i + 2);
		ROUND512(6, 2, 0, 4 * i + 3);
	}

	for(i = 0; i < 8; i++) { 
		H[i] = M[i] ^ midmsg[i];
	}
}

static void Output512(u64 *H, const u64 *M) 
{
	int i;
	u64 midkey[8], midmsg[8];
	u64 tmpkey[2], tmpmsg[2], t[2];

	for(i = 0; i < 8; i++) {
		midkey[i] = H[i];
		midmsg[i] = M[i];
	}

	for(i = 0; i < 8; i++) {
		ROUND512_OUTPUT(4, 0, 6, 4 * i);
		ROUND512_OUTPUT(2, 6, 4, 4 * i + 1);
		ROUND512_OUTPUT(0, 4, 2, 4 * i + 2);
		ROUND512_OUTPUT(6, 2, 0, 4 * i + 3);
	}

	for(i = 0; i < 8; i++) { 
		H[i] = M[i] ^ midmsg[i];
	}
}
